#########################################################################################
#                                                          							              	
#  This R script AveDissR.r was developed in 2017 for assessing genetic distinctness	  
#  and genetic redundancy	 															                                
#                                                          								            
#  Please read "Using AveDissR.pdf" before using this script.      			  
#                                                          				 
#  For more information please contact:                  				 
#                                                      				    	 
#     Dr. Yong-Bi Fu                                  				     	 
#     Agriculture & Agri-Food Canada                 				      	 
#     107 Science Place                            				         
#     Saskatoon SK Canada S7N 0X2                  				        
#     tel: +1-306-385-9298                         				        
#     email: Yong-Bi.Fu@canada.ca    							 
#											 
#  Disclaimer: The R script AveDissR.r and related files are published as it is and	 
#    distributed free of charge and without warranty of any kind, and no guarantee	 
#    is provided as to the suitability or functioning of this software or to the	 
#    results obtained from its use							 
#                                                     				     	 
#########################################################################################
#
# Note: after version v.1.12.+ in data.table, the reshape2 package is not required to be loaded before loading data.table.
# The current vegan package needs loading the two dependent packages of permute and lattice before load vagen.
# The current doParallel package needs  to load two dependent packages of iterators and parallel before load doParallel
# Please install the eight R packages before using this AveDissR.r script, updated at 2021-8-19
#---------------------------------------------------------------------------------------------
 # install.packages("permute",dep=T)
 # install.packages("lattice",dep=T)
 # install.packages("data.table",dep=T)
 # install.packages("vegan",dep=T)
 # install.packages("foreach",dep=T)
 # install.packages("iterators",dep=T)
 # install.packages("doParallel",dep=T)
 # install.packages("matrixStats",dep=T)
 #
 # This new version of AveDissR_Cpdfv5.R was tested on window R4.1 and linux Redhat R4.03 on August 17, 2021
 # Users can set up the number of cores to be used for parallel computing on line 62.
 # The rule of thumb is to use less than half of the cores available.
#########################################################################################

#---------------------
# library packages
#---------------------
library(data.table)
library(permute)
library(lattice)
library(vegan)
library(foreach)
library(iterators)
library(parallel)
library(doParallel)
library(matrixStats)

rm(list=ls())

#----------------------------------------------------------------------------------------------------------------------------------------------------------------------
# 1.1  Setting the running environment for parallel run parameters:   ccs <- makeCluster(10), means there are 10 cores for parallel run in foreach function
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------

NCreads <- 10          # users can set up the number of cores used in the program; the default has 10 cores.

ccs <- makeCluster(NCreads)   
registerDoParallel(ccs)       
setDTthreads(NCreads)     

#-----------------------------------------------------------------------
# Get working directory and file
#----------------------------------------------------------------------- 
path1 <- getwd()
workpath <- list.dirs(path = path1, full.names = TRUE, recursive = TRUE)
cfolders <- basename(workpath)
work3 <- grep("^inputdata$", cfolders, ignore.case = TRUE, value=FALSE)
work4 <- grep("^results$", cfolders, ignore.case = TRUE, value= FALSE)
 
Input_files <- list.files(path = workpath[work3], pattern = NULL, all.files = TRUE,  full.names = TRUE,
                              recursive = TRUE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE)

if (isTRUE(length(Input_files) != 2 )){ 
  stop(print( c("Errors: Input data must be two files, one for parameters setting and the other for marker data input. 
                The parameter setting file should be named with  'parameters_setting'prefixed file name." )))
}

filebase <- basename(Input_files)
PA_index <- grep("parameters_setting", filebase, ignore.case = TRUE, value=FALSE)

#--------------------------------------------------------------------------------------------
# Read the marker data and parameters files into working directory from the inputdata folder
#--------------------------------------------------------------------------------------------
Parameters <- c()
Inpop_markers <- data.table()
bas1 <- c()
seqlabel <- c()

if (isTRUE(length(Input_files) == 2 )){ 
  Parameters_set <- read.csv(file = Input_files[PA_index], header = TRUE, strip.white = TRUE, na.strings = c("NA", NULL), sep=",", fill =TRUE)  # input parameter_setting.csv as a data.frame
  Parameters_set <- data.frame(Parameters_set)
  Parameters <- Parameters_set[1:8, 2]
  
   Inpop_markers <- fread(input = Input_files[-PA_index], header = FALSE, na.strings= c("NA", NULL), strip.white=TRUE, fill=FALSE, blank.lines.skip=FALSE,data.table=getOption("datatable.fread.datatable", TRUE)) 

}
 #Inpop_markers[1:10,1:10,with=FALSE]
#-----------------------------------------------------------------------------------------------
# Analysis types: 1 genetic distinctness, 1; genetic redundancy, 2 
# Five marker data types: dominant marker, 1; haploid SSR or binary marker, 2; haploid SNP, 3; 
#                         diploid SSR with a separator ":" between alleles, 4; and
#                         diploid SNP with no separator between alleles,5.
#-----------------------------------------------------------------------------------------------
Analysis_Type <- as.numeric(Parameters[1])     # 1 Genetic distinctness, 2 genetic redundancy  
Marker_Type <- as.numeric(Parameters[2])
Ind_N <-  as.numeric(Parameters[3])
Pop_N <-  as.numeric(Parameters[4])
Loci_N <- as.numeric(Parameters[5])
Miss_Data <- as.character(Parameters[6], na.rm = TRUE) 
stepwise_selection <- as.numeric(Parameters[7], na.rm = TRUE)
specific_selection <- as.numeric(Parameters[8], na.rm = TRUE)

Ind_Pop_NumberDT <- Inpop_markers[1:2]         # Extract the first two rows to count the sample and population number

Ind_Ns <-  length(unique(unlist(Ind_Pop_NumberDT[1])))-1
Pop_Ns <-  length(unique(unlist(Ind_Pop_NumberDT[2])))-1
Loci_Ns <- Inpop_markers[, .N]-2
 
# Checking the input data consistent with the parameters setting
if(isTRUE(Ind_N != Ind_Ns) | isTRUE(Pop_N != Pop_Ns) | isTRUE(Loci_N != Loci_Ns)) { 
     stop(print(c("Errors: the parameters setting is not consistent with the marker data format.
                   Please check the inputdata folder to format the two inputdata files")))
}

ifelse (isTRUE(is.na(stepwise_selection)),  bas1 <- NA,
         ifelse(stepwise_selection > 0L & stepwise_selection < 1L, bas1 <- round(Ind_Ns * stepwise_selection, 0), 
                                         stop(print(c("Errors: The stepwise selection value should be set between 0 and 1,
                                                      please check the stepwise_selection parameter setting.")))))
 
ifelse (isTRUE(is.na(specific_selection)), seqlabel<- NA,
         ifelse(specific_selection > 0L & specific_selection < 1L, seqlabel <- round(Ind_Ns * specific_selection, 0),
                                        stop(print(c("Errors: The specific selection value should be set between 0 and 1, 
                                                     please check the specific_selection parameter setting.")))))
													 
#-----------------------------------------
# get indexed individual, population name
#-----------------------------------------

Indchar.NewChar<- function(Mtchar, rechar){   # Mtchar is a set of characters, rechar is a character for new order beginning)
  ns1 <- length(Mtchar)
  nc <- nchar(ns1)
  chtc <- paste(rechar, sapply(mapply(rep, nc-nchar(1:ns1), MoreArgs=list(x=0)), paste, collapse=''), 1:ns1,sep="")
  return(chtc)
}


PopNew.name <- function(pops, ch1){     # replace the raw pops vector with its new ordered serial numbered character, ch1 is the new character to be used
  rawPopline <- pops
  IDpo <- as.character(rawPopline)
  uniIDpo <- unique(IDpo)
  uns <- length(uniIDpo)
  unc <- nchar(uns)
  ucht <- paste(ch1, sapply(mapply(rep, unc-nchar(1:uns), MoreArgs=list(x=0)), paste, collapse=''), 1:uns, sep="")
  PopDT <- data.table(rawPopline)
  setnames(PopDT, "Pp1")
  for (i in 1:uns){
    PopDT[Pp1%in%uniIDpo[i], c("Pp2"):= ucht[i]]
  }
   setnames(PopDT, c("Pp1", "Pp2"))
  popw <-PopDT[, Pp2] 
  return(popw)
}

#---------------------------------------------------------------------------
# check marker matrix with missing data and replace with NA (missing data)
#---------------------------------------------------------------------------
headlind.mat.transpose  <- function(Ind.Mark.mat){
  DTM1 <- copy(Ind.Mark.mat)
  rm(Ind.Mark.mat)
  headname <- names(DTM1)
  hn <- length(headname) 
  Hedname <- Indchar.NewChar(1:hn, "H")
  HednameDT <- data.table(t(Hedname))
  HehedmarL <- list( HednameDT, DTM1)
  HehedmarDT <- rbindlist(HehedmarL, use.names=FALSE, fill=FALSE)   # update the rbindlist() in use.names=FALSE to bind by position.
  Hehedn <- HehedmarDT[,.N]
  HehedIDrow <- Indchar.NewChar(1:Hehedn, "Idr")
  RowIDHeHeMar <- data.table(HehedIDrow, HehedmarDT)
  MarHHhe <- transpose(RowIDHeHeMar, fill=NA, ignore.empty=FALSE)
  MHc <- unlist(MarHHhe[2])
  MarHHhesub <- MarHHhe[-c(1:2)]
  setnames(MarHHhesub, MHc)
  saveCols <- MHc[-1] 
  MarHHhesub02s <- MarHHhesub[, ..saveCols]
  setnames(MarHHhesub02s, saveCols)
  return(MarHHhesub02s)
}

Miss.NA.replace <- function(DTo1, MisD, Datt) {               # input data is just for markers, no pop label, Datt is data type
  DTo <- copy(DTo1)
  rm(DTo1)
  if(!is.na(MisD)){
    for (j in names(DTo)){
      ifelse(Datt == 1L | Datt == 2L, set(DTo, which(DTo[[j]]== MisD),j,NA_real_),
             set(DTo, which(DTo[[j]]== MisD), j, NA_character_ ))
    }
  }
  return(DTo)
}

Format.marker <- function(raw.Lmarker.DT, missd, DaTy){
  markr <-copy(raw.Lmarker.DT)
  rm(raw.Lmarker.DT)
  Mnam <- names(markr)
  Lont <- Mnam[-1]
  setnames(markr, c("Ind", Lont ))
  missd <- as.character(missd, na.rm = TRUE)
  chrxc <- function(x){ y <- as.character(x, na.rm = TRUE)}
  markr2 <- markr[ , lapply(.SD,   chrxc), .SDcols= Lont]           # Dominant marker data are coded for 0,1,9(or NA), all are converted into characters
  markr_NA <- Miss.NA.replace(markr2, missd, DaTy)                  # haploid, diploid SSR or SNPs are characters
  return(markr_NA)
}

dip.SSR.SNP.alles.min.max.reorder <- function(DT.Ind.Pop.MK, MarTy){
  DTinPM <- copy(DT.Ind.Pop.MK)
  rm(DT.Ind.Pop.MK)
  narm <- names(DTinPM)
  locAA <- narm[-c(1:4)] 
  Indp4 <- c("Ind","Pop","IndR", "PopR")
  setnames(DTinPM, c(Indp4, locAA))
  IndrP4 <- DTinPM[, ..Indp4]
  setnames(IndrP4, Indp4)
  Indk <- IndrP4[, Ind]
  mKdata <- DTinPM[, ..locAA] 
  mKdata <- data.table(Indk, mKdata)
  setnames( mKdata, c("Ind",locAA ))
  mKdata_melt <- melt(mKdata, id.vars = c("Ind"), measure.vars = list(locAA), value.name=c("alles"))
  ifelse(MarTy==4L, mKdata_melt[ , c("p1","p2"):= tstrsplit(alles, ":", fixed=TRUE)],
                    mKdata_melt[ , c("p1","p2"):= tstrsplit(alles, "", fixed=TRUE)])
  ifelse(MarTy==4L, mKdata_melt[ , c("p1p2"):= paste0(min(p1, p2),":" , max(p1, p2), sep=""), by=c("Ind", "variable")],
                    mKdata_melt[ , c("p1p2"):= paste0(min(p1,p2) , max(p1,p2),sep=""), by=c("Ind", "variable")])
  ifelse(MarTy==4L, mKdata_melt[p1p2%in%c("NA:NA"), c("p1p2"):= NA_character_], mKdata_melt[p1p2%in%c("NANA"), c("p1p2"):= NA_character_])
  mKdata_melt[, c("alles"):= .(p1p2)]
  mKdata_melt[, c("p1", "p2", "p1p2"):= NULL]
  mKdata_dcast <- dcast(mKdata_melt, Ind ~ variable, value.var = c("alles"), value.name= c("loc"))
  mKdata_dcast[, c("Ind"):= NULL]
  dim(mKdata_dcast)
  mKdata_dcast4 <- data.table(IndrP4, mKdata_dcast)
  setnames(mKdata_dcast4, c(Indp4, locAA))
  return(mKdata_dcast4)
}

#---------------------------------------------------------------------------
# Get formatted individual,pop, marker NA to replace matrix data.table (DT)
#---------------------------------------------------------------------------
Raw.trans.Ind.Pop.rename.Mark.Format.NA.replace <- function(Raw.DT.LCIPline.Mar, mIss, Mty){
  Raw.DT.LCIPM <- copy(Raw.DT.LCIPline.Mar)
  rm(Raw.DT.LCIPline.Mar)
  Raw.DT.IP.MTrans <- headlind.mat.transpose(Raw.DT.LCIPM)
  IPLocname <- names(Raw.DT.IP.MTrans)
  Locname <- IPLocname[-c(1:2)]
  setnames(Raw.DT.IP.MTrans,c("Ind", "pop", Locname))
  Rinds <- Raw.DT.IP.MTrans[, Ind]
  Rpops <- Raw.DT.IP.MTrans[, pop] 
  IndRename <- Indchar.NewChar(Rinds, "N")
  PopRename <- PopNew.name(Rpops, "Pop0")
  Ind_indR_Pop_PopR <- data.table(IndRename, PopRename, Rinds, Rpops)
  IndLocn <- c("Ind", Locname)
  LMarkerS <- Raw.DT.IP.MTrans[ , ..IndLocn]
  setnames(LMarkerS, IndLocn)
  FMarkerS <- Format.marker(LMarkerS, mIss, Mty)
  Indpop_markr <- data.table(Ind_indR_Pop_PopR, FMarkerS)
  setnames(Indpop_markr, c("Ind", "Pop","IndR","PopR", Locname))
  if(isTRUE(Mty==4L) | isTRUE(Mty == 5L)) {
    Indpop_markr <- dip.SSR.SNP.alles.min.max.reorder(Indpop_markr, Mty)
  }
  return(Indpop_markr)                                   # output is a transposed DT with 4 columns marker labels
}

#------------------------------------------------------------------------------------------
# To assess the changes in genetic structure, we perform AMOVA and show PCoA plot 
# Analysis type, 1: select a certain number of distinct accessions from a collection
# Analysis type, 2: remove a certain number of redundant accessions from a collection   
#------------------------------------------------------------------------------------------

#---------------------------------------------------------------------
# Dissimilarity matrix calculation; input DT includes ind and markers 
#---------------------------------------------------------------------

DT.dom.hap.codom.AD.mat <- function(DT.ind.domar.mat, cs1, NCRDS1){ 
  cs01 <- makeCluster(cs1)  
  registerDoParallel(cs01)  
  setDTthreads(NCRDS1)
  on.exit(stopCluster(cs01))
  DTtrans <- copy(DT.ind.domar.mat)
  rm(DT.ind.domar.mat)
  hname1 <- names(DTtrans)
  hname1DT <- data.table(t(hname1))
  DTtrans2 <- list(hname1DT, DTtrans)
  DTtrans2DT <- rbindlist(DTtrans2, use.names=FALSE, fill=FALSE) # update the rbindlist() in use.names=FALSE to bind by position.
  locIndts <- headlind.mat.transpose(DTtrans2DT)
  locIndts_names <- names(locIndts)
  InaaA <- locIndts_names[-1] 
  setnames(locIndts, c("Ind", InaaA))
  marInd <- locIndts[, ..InaaA]
  marInName <- names(marInd)
  nid <- length(marInName)
  nr0 <- marInd[,.N]
  ADdis <- matrix(NA, nid, nid)
 B2d <- foreach( k = 1:nid, .combine= rbind, .packages=c('foreach','doParallel', 'data.table')) %dopar% {
    sk1 <- marInName[k]
    sk2 <- marInName[-k]
    A1 <- marInd[, ..sk1]
    a1 <- unlist(A1)
    A2DT <- marInd[, ..sk2]
    hIdm <- names(A2DT)
    hCn <- length(hIdm)
    
    A11DT <- as.data.table(matrix(rep(a1, hCn), nr0, hCn, byrow = FALSE))
    setDF(A11DT)
    setDF(A2DT)
    A112_ueq <- matrix(as.numeric(A11DT!=A2DT, na.rm =TRUE), nr0, nid-1)
    A112_eq <- matrix(as.numeric(A11DT==A2DT, na.rm =TRUE), nr0, nid-1)
    
    uneq <- apply(A112_ueq, 2, sum, na.rm =TRUE)
    eque <- apply(A112_eq, 2, sum,na.rm =TRUE )
    B1 <- uneq/(uneq+eque)
  }
  nn1 <- ncol(B2d)
  for(k in 1: (nn1+1)){
    ifelse(k == 1, ADdis[k, ]<- c(NA, B2d[k,]), 
           ifelse ( 1 < k & k < nid, ADdis[k, ] <- c(B2d[k, 1:(k-1)], NA, B2d[k, k:nn1]),
                    ADdis [k, ] <-c(B2d[k, ], NA)))    
  }
  return(ADdis)
}

#----------------------------------------------------
# Order AD descending from high to low, and index the new Ind with new names from AD001-AD00n
# ---------------------------------------------------

DT.AD.desced.index.random <- function(AD.ind.pop.indr.popr.marker){     #  input data is a DT with AD, ind, pop and marker
    dt.ad <- copy(AD.ind.pop.indr.popr.marker)
    dtname <- names(dt.ad)
    locnn <- dtname[-c(1:5)]
    setnames(dt.ad, c("AD","Ind","Pop","IndR","PopR",locnn))
    desdt.AD <- dt.ad[order(-AD)]
    mn <- desdt.AD[,.N]
    adInd <-Indchar.NewChar(1:mn, "AD")
    adInd.Popr.des.AD.dt <- data.table(adInd, desdt.AD)
    setcolorder(adInd.Popr.des.AD.dt, c("adInd","AD","Ind","Pop","IndR","PopR",locnn))
    return(adInd.Popr.des.AD.dt)                               #  Output has 6 columns for new indexed adInd order and random popr, used for further analysis 
}
 
AD.count.plot <- function(AD.value, dtyy,analy){
   value0 <- as.numeric(AD.value)
   my_hist <- hist(value0, breaks = 20, plot = F, right= TRUE)
   my_breaks <- my_hist$breaks
   my_counts <- my_hist$counts
   myint <- my_breaks[2]-my_breaks[1]
   myn <- length(my_breaks)
   myx0 <- my_breaks[1] - myint
   myxn <- my_breaks[myn] + myint
   mybreaks2 <- c(myx0, my_breaks)
   mycout2 <- c(0, my_counts, 0)
   valcount <- data.frame(mybreaks2,mycout2)
   nbreak <- nrow(valcount)
   colnames(valcount) <- c("AD","count")
   pdf(file=paste(dtyy,"_",analy,"_",nbreak,"_individual_average_dissimilarity_frequecy_distribution", ".pdf", sep= "" ),
       width=7, height=7, paper="special", fonts="Helvetica")
   mytitle = paste("Frequency distribution of individual average dissimilarity")
   ymax <- max(valcount$count)
   yseq <- c()
   ifelse(ymax <= 10, yseq <- seq(0,ymax+1, 1),
          ifelse(ymax <= 50, yseq <- seq(0,ymax+2, 2),
                 ifelse(ymax <= 100, yseq <- seq(0, ymax+10, 10),
                        ifelse(ymax <= 500, yseq <- seq(0,ymax+50, 50),
                               ifelse(ymax <=1000, yseq <- seq(0, ymax+200, 200),
                                     ifelse(ymax <= 2000, yseq <- seq(0, ymax+400, 400),
                                            ifelse(ymax <= 5000, yseq <- seq(0, ymax+500, 500),
                                                   ifelse(ymax <= 10000, yseq <- seq(0, ymax+1000, 1000),
                                                          ifelse(ymax <= 20000, yseq <- seq(0, ymax+2000, 2000),
                                                                 ifelse(ymax <= 30000, yseq <- seq(0, ymax+3000, 3000),
                                                                        ifelse(ymax <= 40000, yseq <- seq(0, ymax+4000, 4000),
                                                                               ifelse(ymax <= 50000, yseq <- seq(0, ymax+5000, 5000),
                                                                                      ifelse(ymax <= 60000, yseq <- seq(0, ymax+6000, 6000),
                                                                                             ifelse(ymax <= 70000, yseq <- seq(0, ymax+7000, 7000),
                                                                                                    ifelse(ymax <= 80000, yseq <- seq(0, ymax+8000, 8000),
                                                                                                           ifelse(ymax <= 90000, yseq <- seq(0, ymax+9000, 9000),
                                                                                                                  ifelse(ymax <= 100000, yseq <- seq(0, ymax+10000, 10000),
                                                                                                                       yseq <- seq(0, ymax+20000, 20000))))))))))))))))))
   myplot <- barplot(valcount$count, axes=F, main="Frequency distribution of individual average dissimilarity", 
                     xlab =" Average dissimilarity for a sample", ylab = "Sample counts",cex.names= 1.0, ylim=c(0, 1.2*ymax))
   axis(1,at=myplot, labels= mybreaks2)
   axis(2,yseq)
   dev.off()
 }

#-------------------------------------------------------------------
# AMOVA calculation for dominant and haploid data 
#--------------------------------------------------------------------

##################################
# for convert the 2 alleles co-dominant SNP marker's genotypes as 1 2 3 and 9 (AA Aa aa and missing data) for AMOVA calculation

Codom.SNP.alle.convert.ATCG.to.number1239.fun <- function(Dt.adind.pop.SNP.marker, ccs0, Threadn0){
  
  cs02 <- makeCluster(ccs0)    
  registerDoParallel(cs02)     
  setDTthreads(Threadn0)
  on.exit(stopCluster(cs02))
  
  Ssmat <- copy(Dt.adind.pop.SNP.marker)
  rm(Dt.adind.pop.SNP.marker)
  DTnames <- names(Ssmat)
  LocAB <- DTnames[-c(1:2)]
  setnames(Ssmat, c("adInd","Pop",LocAB))
  Indt <- Ssmat[, adInd]
  Popt <- Ssmat[, Pop] 
  setkey(Ssmat, "adInd")
  setindex(Ssmat, adInd)
  Locsn <- length(LocAB) 
  DTnam <- names(Ssmat)
 
  groupsID <- ceiling(Locsn/(2000*Threadn0))
  
  codeMatDTcbind1 <- data.table()
  
  for (j in 1:groupsID){                   # The gropuID to speed up the data conversion with large marker matrix using foreach() for paralle
    
    scn1 <- (j-1)*2000*Threadn0+3
    ifelse (isTRUE(j*2000*Threadn0 < Locsn), scn2 <- j*2000*Threadn0+2, scn2 <- Locsn+2)
    LcAsm <- c(DTnam[1], DTnam[scn1:scn2])
    
    scn12Mat <- Ssmat[ , ..LcAsm]
    scNme <- names(scn12Mat)
    Locs1 <- scNme[-1]
    scnn <- length(Locs1)
    setnames(scn12Mat, c("adInd", Locs1))
    
    codeMat0 <- c()
    
    codeMat0 <-foreach (i = 1:scnn, .combine= 'cbind', .inorder= TRUE, .packages = c("data.table", "foreach", "doParallel")) %dopar% {
        IdnLc1 <- c(DTnam[1], Locs1[i])
        dt0 <- scn12Mat[ , ..IdnLc1]
        setnames(dt0, c("Id","Lc")); setkey(dt0, Id); setindex(dt0, Id)
        dt0[, c("p1","p2") := tstrsplit(Lc, "", fixed =TRUE)]
        p1 <- dt0[, p1];  p2 <- dt0[, p2]; p12 <- c(p1, p2)
        p12u <- unique(p12[!is.na(p12)])
        a1 <- length(p12[p12%in%p12u[1]])
        a2 <- length(p12[p12%in%p12u[2]])
        AA <- c() ;  BB<- c();  AB1 <- c();  AB2 <- c()
        ifelse(isTRUE(a1>=a2), AA <- paste(p12u[1], p12u[1], sep = ""), AA <- paste(p12u[2], p12u[2], sep = "") )
        ifelse(isTRUE(a1>=a2), BB <- paste(p12u[2], p12u[2], sep = ""), BB <- paste(p12u[1], p12u[1], sep = "") ) 
        ifelse(isTRUE(a1>=a2), AB1 <- paste(p12u[1], p12u[2], sep = ""), AB1 <- paste(p12u[2], p12u[1], sep = ""))
        ifelse(isTRUE(a1>=a2), AB2 <- paste(p12u[2], p12u[1], sep = ""), AB2 <- paste(p12u[1], p12u[2], sep = ""))
        dt0[Lc%in%AA, c("code"):= 3L]                    # code AA as "3"
        dt0[Lc%in%BB, c("code"):= 1L]                    # code aa as "1"
        dt0[Lc%in%AB1 |Lc%in%AB2 , c("code"):= 2L]       # code Aa as "2"
        dt0[ ,  c("code2"):= .(code)]
        dt0[is.na(code),  c("code2"):= 9L]       # code missing data as "9"
        code2 <- dt0[ , code2]    
        code3 <- c(Locs1[i], code2)
    }
       LoIndt <- c("Inoc", Indt);  LoPop <- c("Poc", Popt)
       ifelse(isTRUE(j==1), codeMatDTcbind1 <- data.table(LoIndt, LoPop, as.data.frame(codeMat0)),
                 codeMatDTcbind1 <- data.table(codeMatDTcbind1, as.data.frame(codeMat0)))   # for groupID j converted data 1239 saved on the cbind data.table, 
      }
  
      setnames(codeMatDTcbind1, DTnam)
      return(codeMatDTcbind1)
}

#----------------------------------

DT.dom.hap.SSR.SNP.amova.fun <- function(Dt.adind.pop.marker, cs2, NCRDS2){  # Dt.ind.pop.marker is a DT with Ind, Pop, markers
  
  Dtr <- copy(Dt.adind.pop.marker)                             
  rm(Dt.adind.pop.marker)
  nrw <- Dtr[, .N]
  nc2 <- length(names(Dtr))-2
  loc2n <-Indchar.NewChar(1:nc2, "loc")
  setnames(Dtr, c("Ind", "Pop", loc2n))
  setkey(Dtr, "Pop")                         # Dtr is DT with Ind, Pop and markers, sorted by Ind Pop
  
  DT.dom.dis.mat <- function(one.pop.IM, kcs2, kNCRDS2){
    cs02 <- makeCluster(kcs2)    
    registerDoParallel(cs02)     
    setDTthreads(kNCRDS2)
    on.exit(stopCluster(cs02))
    DTt <- copy(one.pop.IM)
    rm(one.pop.IM)
    IMnam <- names(DTt)
    IMnamDT <- data.table(t(IMnam))
    DTt2L <- list(IMnamDT, DTt)
    DTt2LDT <- rbindlist(DTt2L, use.names=FALSE, fill=FALSE) 
    MIndts <- headlind.mat.transpose(DTt2LDT)
    IndsName <- names(MIndts) 
    Indscol <- IndsName[-1]
    setnames(MIndts, c("Loc", Indscol))
    Marks <- MIndts[, ..Indscol]
    MarId <- names(Marks)
    setnames(Marks, MarId)
    dc1 <- length(MarId)
    dr1 <- Marks[, .N] 
    st <-0
    tb <- c()
    tb <- foreach( k = 1:dc1, .combine= 'c', .packages=c('data.table','foreach','iterators','parallel','doParallel')) %dopar% {
      Idss <- MarId
      AAaeM <- copy(Marks)
      d1k <- Idss[k]
      d1 <- AAaeM[, ..d1k]
      d1v <- unlist(d1)
      BBaeM <- as.data.table(matrix(rep(d1v, dc1), dr1, dc1, byrow = FALSE))
      setDF(AAaeM)
      setDF(BBaeM)
      ABueqM <- apply(AAaeM != BBaeM, 2, as.numeric, na.rm=TRUE)
      uequal <- sum(ABueqM, na.rm= TRUE)/(2*dc1)
      uequal
    }
    st <- sum(tb, na.rm=TRUE)
    return(st)
  }
  
  ssw <- 0
  npm <- unique(Dtr$Pop)
  np <- length(npm)
  nx <- rep(0, np)
  
  for (np1 in 1:np){
    Dtrp0 <- copy(Dtr)
    setkey(Dtrp0, "Pop")
    Dtrp1 <- Dtrp0[.(npm[np1]), ]
    rm(Dtrp0)
    nn1 <- Dtrp1[, .N ]
    setcolorder(Dtrp1, c("Ind", "Pop", loc2n))
    Dtrp1IM <- Dtrp1[, c("Pop"):= NULL]
    ncount <- DT.dom.dis.mat(Dtrp1IM, cs2, NCRDS2)
    ssw <- ssw + ncount
    nx[np1]<- nn1
  }
  
  DtrT <- copy(Dtr)
  setkey(DtrT, "Ind")
  DtrwT <- DtrT[, c("Pop"):= NULL] 
  sst <- DT.dom.dis.mat(DtrwT, cs2, NCRDS2)
  ssa <- sst-ssw
  nx2 <- ((sum(nx))^2-sum(nx^2))/((np-1)*sum(nx))
  nw <- nrw - np
  vw <- ssw/nw
  va <- (ssa/(np-1)-vw)/nx2
  pva <- va/(va+vw)
  pvw <- 1-pva
  amova.out <- cbind(nrw, np, nw, ssa, ssw, va, vw, pva, pvw)
  return(amova.out)
}

#------------------------------------------
# AMOVA calculation for SSR and SNP data
#------------------------------------------

DT.codom.mutiple.alles.SSR.or.SNP.genotype.amova.fun <- function(Dt.adind.pop.ssr.marker, Matye, cs3, NCRDS3 ){  # ind.pop.snp.marker is a DT with Ind, Pop, markers wide data.table
  
  Dtr <- copy(Dt.adind.pop.ssr.marker)  # Dtr is DT with Ind, Pop and markers, sorted by Pop 
  rm(Dt.adind.pop.ssr.marker)
  nrw <- Dtr[, .N]
  nc2 <- length(names(Dtr))-2
  loc2n <-Indchar.NewChar(1:nc2, "loc") 
  setnames(Dtr, c("Ind", "Pop", loc2n))
  setkey(Dtr, "Pop")                          # keyed by Pop 
  
   DT.codom.dis.mat <- function(one.pop.indM, Matye45, cs345, NCRDS345){
    
    cs03 <- makeCluster(cs345)                
    registerDoParallel(cs03)                 
    setDTthreads(NCRDS345)
    on.exit(stopCluster(cs03))
    
    IndLCmat <- copy(one.pop.indM)
    rm(one.pop.indM)
    IdLCnam <- names(IndLCmat)
    IdLCnamDT <- data.table(t(IdLCnam))
    IndIndLCmatL <- list(IdLCnamDT,IndLCmat)
    DTIndLCmat1 <- rbindlist(IndIndLCmatL, use.names=FALSE, fill=FALSE) # update the rbindlist() in use.names=FALSE to bind by position.
    MsIndt <- headlind.mat.transpose(DTIndLCmat1)
    MsIndNm<- names(MsIndt)
    Indcols <- MsIndNm[-1] 
    setnames(MsIndt, c("Loc",Indcols))
    Markd <- MsIndt[ , ..Indcols] 
    LN <- MsIndt[, .(Loc)]
    drn <- MsIndt[ , .N]
    dcn <- length(Indcols) 
    
    LcId_aLLeN <- drn*dcn
    LLnT <- data.table(Indchar.NewChar(1:LcId_aLLeN, "LL"))
    
    A1A2 <- data.table(LN, Markd)       # Extract the two alles in the target matrix A1A1 for comparison use
    setnames(A1A2, c("Loc", Indcols))
    A1A2_melt <- melt(A1A2, measure= list(Indcols), value.name = "Aalle" )
    rm(A1A2)
    if(isTRUE(Matye45==4L)) {A1A2_melt[ , c("Ap1","Ap2"):= tstrsplit(Aalle, ":", fixed=TRUE)]}
    if(isTRUE(Matye45==5L)) {A1A2_melt[ , c("Ap1","Ap2"):= tstrsplit(Aalle, "", fixed=TRUE)]}
    A1A2_melt[, c("Aalle"):= NULL]
    A1A2_meltcast <- dcast(A1A2_melt, Loc ~ variable, value.var = c("Ap1","Ap2"))
    rm(A1A2_melt)
    A1A2_meltcastNm <- names(A1A2_meltcast)
    setnames(A1A2_meltcast, c("ALoc", A1A2_meltcastNm[-1]))
    setkey(A1A2_meltcast, "ALoc")
    Ap1C <- grep("^Ap1", A1A2_meltcastNm, ignore.case = TRUE, value=TRUE) 
    Ap2C <- grep("^Ap2", A1A2_meltcastNm, ignore.case = TRUE, value=TRUE)  
    Ap1alle <- setDF(A1A2_meltcast[, ..Ap1C]) 
    Ap2alle <- setDF(A1A2_meltcast[, ..Ap2C]) 
    
    IDkd <- names(Markd)
    st <- 0 
    tb <- c()
    tb <- foreach (k = 1:dcn, .combine= 'c', .packages=c('data.table','foreach','iterators','parallel','doParallel')) %dopar% {
      IDkd1 <- IDkd[k]
      d1s <- Markd[ , ..IDkd1]
      d1sV <- unlist(d1s)
      B1B2 <- as.data.table(matrix(rep(d1sV, dcn), drn, dcn, byrow = FALSE))
      LB1B2 <- data.table(LN, B1B2)
      d1mc <- paste("d1", IDkd, sep="" )   # Extract the two alles in the target matrix B1B2 for comparison use
      setnames(LB1B2, c("Loc",d1mc)) 
      
      B1B2_melt <- melt(LB1B2, measure= list(d1mc), value.name = "Balle" )
      rm(LB1B2)
      if(isTRUE(Matye45==4L)) {B1B2_melt[ , c("Bp1","Bp2"):= tstrsplit(Balle, ":", fixed=TRUE)]}
      if(isTRUE(Matye45==5L)) {B1B2_melt[ , c("Bp1","Bp2"):= tstrsplit(Balle, "", fixed=TRUE)]}
      B1B2_melt[, c("Balle"):= NULL]
      B1B2_meltcast <- dcast(B1B2_melt, Loc ~ variable, value.var = c("Bp1","Bp2"))
      rm(B1B2_melt)
      B1B2_meltcastNm <- names(B1B2_meltcast)
      setnames(B1B2_meltcast, c("BLoc", B1B2_meltcastNm[-1]))
      setkey(B1B2_meltcast, "BLoc")
      Bp1C <- grep("^Bp1", B1B2_meltcastNm, ignore.case = TRUE, value=TRUE) 
      Bp2C <- grep("^Bp2", B1B2_meltcastNm, ignore.case = TRUE, value=TRUE)  
      Bp1alle <- setDF(B1B2_meltcast[, ..Bp1C]) 
      Bp2alle <- setDF(B1B2_meltcast[, ..Bp2C]) 
      
      ## Construct Ap1Ap2, Bp1Bp2, Ap1Bp1, Ap1Bp2, Ap2Bp1, Ap2Bp2 equal or unequal matrix for use to convert code
      Ap1Ap2 <- apply(Ap1alle == Ap2alle, 2, as.numeric, na.rm=TRUE)
      LAp1Ap2 <- data.table(LN, Ap1Ap2)
      AAnm <- names(LAp1Ap2)
      A1A2p <- paste0("AA_", AAnm[-1], sep="")
      setnames(LAp1Ap2, c("Loc", A1A2p))
      
      Bp1Bp2 <- apply(Bp1alle == Bp2alle, 2, as.numeric, na.rm=TRUE)
      LBp1Bp2 <- data.table(LN, Bp1Bp2)
      BBnm <- names(LBp1Bp2)
      B1B2p <- paste0("BB_", BBnm[-1], sep="")
      setnames(LBp1Bp2, c("Loc", B1B2p))
      
      Ap1Bp1 <- apply(Ap1alle == Bp1alle, 2, as.numeric, na.rm=TRUE)
      LAp1Bp1 <- data.table(LN, Ap1Bp1)
      A1B1nm <- names(LAp1Bp1)
      A1B1p <- paste0("A1B1_", A1B1nm[-1], sep="")
      setnames(LAp1Bp1, c("Loc", A1B1p))
    
      Ap1Bp2 <- apply(Ap1alle == Bp2alle, 2, as.numeric, na.rm=TRUE)
      LAp1Bp2 <- data.table(LN, Ap1Bp2)
      A1B2nm <- names(LAp1Bp2)
      A1B2p <- paste0("A1B2_", A1B2nm[-1], sep="")
      setnames(LAp1Bp2, c("Loc", A1B2p))
      
      Ap2Bp1 <- apply(Ap2alle == Bp1alle, 2, as.numeric, na.rm=TRUE)
      LAp2Bp1 <- data.table(LN, Ap2Bp1)
      A2B1nm <- names(LAp2Bp1)
      A2B1p <- paste0("A2B1_", A2B1nm[-1], sep="")
      setnames(LAp2Bp1, c("Loc", A2B1p))
      
      Ap2Bp2 <- apply(Ap2alle == Bp2alle, 2, as.numeric, na.rm=TRUE)
      LAp2Bp2 <- data.table(LN, Ap2Bp2)
      A2B2nm <- names(LAp2Bp2)
      A2B2p <- paste0("A2B2_", A2B2nm[-1], sep="")
      setnames(LAp2Bp2, c("Loc", A2B2p))
      
      LAp1Ap2_melt <- melt(LAp1Ap2, measure= list(A1A2p), value.name = "AAalle")
      LAp1Ap2_melt[1:5]
      LC_Indn <- names(LAp1Ap2_melt)[1:2]
      LC_IndnMat <- LAp1Ap2_melt[, ..LC_Indn] 
      AAalle <- names(LAp1Ap2_melt)[-c(1:2)]
      AAalleM <- LAp1Ap2_melt[, ..AAalle]
      
      LBp1Bp2_melt <- melt(LBp1Bp2, measure= list(B1B2p), value.name = "BBalle")
      BBalle <- names(LBp1Bp2_melt)[-c(1:2)]
      BBalleM <- LBp1Bp2_melt[, ..BBalle]
      
      LAp1Bp1_melt <- melt(LAp1Bp1, measure= list(A1B1p), value.name = "A1B1alle")
      A1B1alle <- names(LAp1Bp1_melt)[-c(1:2)]
      A1B1alleM <- LAp1Bp1_melt[, ..A1B1alle]
      
      LAp1Bp2_melt <- melt(LAp1Bp2, measure= list(A1B2p), value.name = "A1B2alle")
      A1B2alle <- names(LAp1Bp2_melt)[-c(1:2)]
      A1B2alleM <- LAp1Bp2_melt[, ..A1B2alle]
      
      LAp2Bp1_melt <- melt(LAp2Bp1, measure= list(A2B1p), value.name = "A2B1alle")
      A2B1alle <- names(LAp2Bp1_melt)[-c(1:2)]
      A2B1alleM <- LAp2Bp1_melt[, ..A2B1alle]
      
      LAp2Bp2_melt <- melt(LAp2Bp2, measure= list(A2B2p), value.name = "A2B2alle")
      A2B2alle <- names(LAp2Bp2_melt)[-c(1:2)]
      A2B2alleM <- LAp2Bp2_melt[, ..A2B2alle]
      
      mixAABBAB4 <- data.table(LLnT, LC_IndnMat, AAalleM, BBalleM, A1B1alleM, A1B2alleM, A2B1alleM, A2B2alleM)
      setnames(mixAABBAB4, c("LL", LC_Indn, "AA", "BB", "A1B1", "A1B2", "A2B1", "A2B2"))
      setkey(mixAABBAB4, "LL")
      
      mixAABBAB4[AA==1L & BB==1L & A1B1==1L & A2B2==1L , c("code"):= 0L]                                           # d^2(ii,ii)==0
      mixAABBAB4[AA==0L & BB==0L & A1B1==1L & A2B2==1L | AA==0L & BB==0L & A1B2==1L & A2B1==1L , c("code"):= 0L]  # d^2(ij,ij)==0
      mixAABBAB4[AA==1L & BB==0L & A1B1==1L & A2B2==0L | AA==1L & BB==0L & A1B1==0L & A2B2==1L , c("code"):= 1L]  # d^2(ii,ij)==1,d^2(ii,ji)==1
      mixAABBAB4[AA==0L & BB==1L & A1B1==1L & A2B2==0L | AA==0L & BB==1L & A1B1==0L & A2B2==1L , c("code"):= 1L]  # d^2(ij,ii)==1,d^2(ij,jj)==1
      mixAABBAB4[AA==0L & BB==0L & A1B1==1L & A2B2==0L | AA==0L & BB==0L & A1B1==0L & A2B2==1L | AA==0L & BB==0L & A2B1==1L & A1B2==0L, c("code"):= 1L]  # d^2(ij,ik)==1,d^2(ij,kj)==1
      mixAABBAB4[AA==1L & BB==0L & A1B1==0L & A2B2==0L | AA==0L & BB==1L & A1B1==0L & A2B2==0L , c("code"):= 3L]  # d^2(ii,jk)==3,d^2(jk,ii)==3
      mixAABBAB4[AA==0L & BB==0L & A1B1==0L & A1B2==0L & A2B1 ==0L & A2B2==0L, c("code"):= 2L]                    # d^2(ij,kh)==2
      mixAABBAB4[AA==1L & BB==1L & A1B1==0L & A1B2==0L & A2B1==0L & A2B2==0L, c("code"):= 4L]                     # d^2(ii, jj)==4
      mixAABBAB4[ , c("code2"):= .(code)]
      mixAABBAB4[is.na(code), c("code2"):= 0L]
      
      setnames(mixAABBAB4, c("LL","Loc", "variable","AA", "BB", "A1B1", "A1B2", "A2B1", "A2B2","code","code2" ))
      mixAABBAB4_dcast <- dcast(mixAABBAB4, Loc~variable, value.var = c("code2"))
      ChrNum <- function(x){y <- as.numeric(x, na.rm=TRUE); return(y)}
      mixAABBAB4_dcast2 <- mixAABBAB4_dcast[ , lapply(.SD, ChrNum), .SDcols = -c("Loc")]
      setDF(mixAABBAB4_dcast2)
      
      tb1 <- sum(mixAABBAB4_dcast2, na.rm=TRUE)/(2*dcn)  #  0810
      tb1
    }
    st <- sum(tb, na.rm=TRUE) 
    return(st)
  }
  
  ssw <- 0
  npm <- unique(Dtr$Pop)
  np <- length(npm)
  nx <- rep(0, np,1)    # 812 
  for (np1 in 1:np){
    Dtrp0 <- copy(Dtr)
    setkey(Dtrp0, "Pop")
    Dtrp1 <- Dtrp0[.(npm[np1]), ]
    rm(Dtrp0)
    nn1 <- Dtrp1[, .N ]
    setcolorder(Dtrp1, c("Ind", "Pop", loc2n))
    Dtrp1[, c("Pop"):=NULL]
    ncount <- DT.codom.dis.mat(Dtrp1, Matye, cs3, NCRDS3)
    rm(Dtrp1)
    ssw <- ssw + ncount
    nx[np1] <- nn1
  }
  
  keyw <- c("Ind","Pop")
  setkeyv(Dtr, keyw)
  Dtrwpp <-Dtr[, c("Pop"):=NULL]
  sst <- DT.codom.dis.mat(Dtrwpp, Matye, cs3, NCRDS3)
  ssa <- sst-ssw
  nx2 <- ((sum(nx))^2-sum(nx^2))/((np-1)*sum(nx))
  nw <- nrw - np
  vw <- ssw/nw
  va <- (ssa/(np-1)-vw)/nx2
  pva <- va/(va+vw)
  pvw <- 1-pva
  amova.out <- cbind(nrw, np, nw, ssa, ssw, va, vw, pva, pvw)
  return(amova.out)
}

#----------------------
#for 2 allele co-dominant SNP marker with genotypes as 1 2 3 and 9 (AA Aa aa and missing data) AMOVA calculation

DT.dip.codom.2.alle.SNP.Amova.foreach.fun <- function (inputdata, kcs4, kNCRDS4) {
  odata1 <- copy(inputdata)
  rm(inputdata)
  nrw <- odata1[ , .N]
  Odname <- names(odata1)
  loc2n <- Odname[-c(1:2)]
  setnames(odata1, c("Ind", "Pop",  loc2n))
  LocTn <- length(loc2n)
  
  ds.function <- function (indata, kcs3, kNCRDS3) { 
    
    kcs03 <- makeCluster(kcs3)                
    registerDoParallel(kcs03)                 
    setDTthreads(kNCRDS3)
    on.exit(stopCluster(kcs03))
    
    indataM <- copy(indata)
    rm(indata)
    IDSn <- indataM[, .N]
    IdLCname <- names(indataM)
    IdLcSn <- length(IdLCname) 
    LcNm <- IdLCname[-1] 
    LcNmSn <- IdLcSn-1
    
    IdPWise_aLLeN <- LcNmSn*(IDSn-1)
    LLn <- Indchar.NewChar(1:IdPWise_aLLeN, "LL")
    LLnT <- data.table(LLn)
    
    setnames(indataM, c("Ind", LcNm))
    IDSndT <- Indchar.NewChar(1:IDSn, "Nr")
    indataM1 <- data.table(IDSndT,indataM)
    rm(indataM)
    hcsd <- names(indataM1)
    hcsdT <- data.table(t(hcsd))
    IndindataM1L <- list(hcsdT,indataM1)
    DTindataM2 <- rbindlist(IndindataM1L, use.names=FALSE, fill=FALSE) # update the rbindlist() in use.names=FALSE to bind by position.
    rm(IndindataM1L,indataM1)
    MIndt1 <- headlind.mat.transpose(DTindataM2)
    rm(DTindataM2)
    
    MIndNm <- unlist(MIndt1[1])
    MIndt2 <-  MIndt1[-1]
    setnames(MIndt2, MIndNm)
    rm(MIndt1)
    Indscol <- MIndNm[-1]
    setnames(MIndt2, c("Loc", Indscol))
    LNc <- MIndt2[, Loc]
    LN <- data.table(Indchar.NewChar(LNc, "Lc"))
    Makd <- MIndt2[, ..Indscol]
    rm(MIndt2)
    
    drn <-  LN[, .N]
    IDme <- names(Makd)
    dcn <- length(IDme)
    st <- 0 
    tb <- c()
    tb <- foreach (k = 1:dcn, .combine= 'c', .packages=c('data.table','foreach','iterators','parallel','doParallel')) %dopar% {
      
      IDme1 <- IDme[k]
      IDme2 <- IDme[-k]
      d1 <- Makd[ , ..IDme1]
      d1v <- unlist(d1)
      d2 <- Makd[, ..IDme2]
      hdm <- names(d2)
      d2cn <- length(hdm)
      d2rn <- d2[, .N]
      d11 <- as.data.table(matrix(rep(d1v, d2cn), d2rn, d2cn, byrow = FALSE))
      hmc <- paste("d", hdm, sep="" )
      setnames(d11, hmc) 
      Ld11 <- data.table(LN, d11)
      setnames(Ld11, c("Loc", hmc))
      Ld11_melt <- melt(Ld11, measure= list(hmc), value.name = "Ld11alle" )
      Ld11allenam <- c("Loc", "variable", "Ld11alle")
      setnames(Ld11_melt, Ld11allenam)
      Ld11alle <- Ld11allenam[3]
      Ld11alleML <-  Ld11_melt[, ..Ld11alle]
      
      Ld2 <- data.table(LN, d2)
      setnames(Ld2, c("Loc", hdm))
      Ld2_melt <- melt(Ld2, measure= list(hdm), value.name = "Ld2alle" )
      Ld2allenam <- c("Loc", "variable", "Ld2alle")
      setnames(Ld2_melt, Ld2allenam)
      Ld2alle <-Ld2allenam[3] 
      Ld2alleML <-  Ld2_melt[, ..Ld2alle]
      
      MixLd11d2melt <- data.table( LLnT, Ld11_melt, Ld2alleML)
      hed5nm <- c("LLn","Loc", "variable","Ld11alle", "Ld2alle")
      setnames(MixLd11d2melt, hed5nm)
      Loc_variable <- hed5nm[2:3]
      Loc_variableDT <- MixLd11d2melt[, ..Loc_variable]
      
      ChrNumer <- function(x){y <- as.numeric(x, na.rm=TRUE); return(y)}
      MixLd11d2melt_Two <- MixLd11d2melt[ , lapply(.SD, ChrNumer), .SDcols = -c("LLn", "Loc", "variable")]
      SLLnLC <- c("LLn","Ld11alle", "Ld2alle") 
      setnames(MixLd11d2melt_Two, SLLnLC[-1])
      
      MixLd11d2melt_Two <- data.table( LLnT, MixLd11d2melt_Two)
      setnames(MixLd11d2melt_Two, SLLnLC)
      setkey(MixLd11d2melt_Two, "LLn")
      
      MixLd11d2melt_Two[Ld11alle != Ld2alle & Ld11alle < 9L & Ld2alle < 9L, c("code"):= 1L]
      MixLd11d2melt_Two[Ld11alle != Ld2alle & Ld11alle < 9L & Ld2alle < 9L & Ld11alle == 1L & Ld2alle == 3L, c("code"):= 4L] 
      MixLd11d2melt_Two[Ld11alle != Ld2alle & Ld11alle < 9L & Ld2alle < 9L & Ld11alle == 3L & Ld2alle == 1L, c("code"):= 4L] 
      MixLd11d2melt_Two[ , c("code2"):= .(code), by= LLn]
      MixLd11d2melt_Two[is.na(code), c("code2"):= 0L]
      
      Loc_MixLd11d2melt_Two <- data.table(Loc_variableDT, MixLd11d2melt_Two)
      setnames(Loc_MixLd11d2melt_Two, c("Loc", "variable","LLn","Ld11alle", "Ld2alle", "code","code2"))
      
      Loc_MixLd11d2melt_Two_dcast <- dcast(Loc_MixLd11d2melt_Two, Loc~variable, value.var = c("code2"))
      Loc_MixLd11d2melt_Two_dcast2 <- Loc_MixLd11d2melt_Two_dcast[ , lapply(.SD, ChrNumer), .SDcols = -c("Loc")]
      
      setDF(Loc_MixLd11d2melt_Two_dcast2)
      
      tb1 <- sum(Loc_MixLd11d2melt_Two_dcast2, na.rm=TRUE)/(2*dcn)  #  0811
      tb1
    }
    st <- sum(tb, na.rm=TRUE) 
    return(st)
  }
  
  ssw <- 0
  npm <- unique(odata1$Pop)
  np <- length(npm)
  nx <- rep(0, np,1)
  for (np1 in 1:np){
    Dtrp0 <- copy(odata1)
    setkey(Dtrp0, "Pop")
    Dtrp1 <- Dtrp0[.(npm[np1]), ]
    rm(Dtrp0)
    nn1 <- Dtrp1[, .N ]
    setcolorder(Dtrp1, c("Ind", "Pop", loc2n))
    Dtrp1[, c("Pop"):=NULL]
    ncount <- ds.function(Dtrp1, kcs4, kNCRDS4)
    rm(Dtrp1)
    ssw <- ssw + ncount
    nx[np1] <- nn1
  }
  
  odata12 <- odata1[, c("Pop"):=NULL]
  sst <-  ds.function(odata12, kcs4, kNCRDS4)
  ssa <- sst-ssw 
  nx2 <- ((sum(nx))^2-sum(nx^2))/((np-1)*sum(nx))
  nw <-nrow(odata1)-np
  vw <- ssw/nw
  va <- (ssa/(np-1)-vw)/nx2
  pva <- va/(va+vw)
  pvw <- 1-pva
  amova.out <- cbind(nrw, np, nw, ssa, ssw, va, vw, pva, pvw)
  return(amova.out)
}
  

#-----------------------------------------------------------------------------
# Select high AD sampleS and do amova to assess change in genetic structure
#-----------------------------------------------------------------------------  

DT.simu.high.AD.stepwise.selected.amova <- function(DT.adind.popr.markers, basel, Dyt, anTY, cs4, NCRDS4){
 
  DT.pop.refine <- function(Dtr.ind.pop.mar){
    Dtorr <- copy(Dtr.ind.pop.mar)
    nrc1 <- length(names(Dtorr))-2
    loc1n <- Indchar.NewChar(1:nrc1, "loc") 
    setnames(Dtorr, c("Ind", "Pop", loc1n))
    setkey(Dtorr, "Pop")
    DtorrN <- Dtorr[, .N, by = "Pop"][N>1L]   # filter out the data with only single population for amova analysis
    DtorrN[, c("N") := NULL]
    Dtorref <- merge(DtorrN, Dtorr, by = "Pop", all.x=TRUE) 
    setcolorder(Dtorref, c("Ind","Pop",loc1n))
    setkey(Dtorref, "Ind")
    return(Dtorref)
  }
  
  dtp <- copy(DT.adind.popr.markers)
  dtp[1:10,1:10,with=FALSE]
  nrp <- dtp[, .N]
  ncc1 <- length(names(dtp))-2
  locc1n <- Indchar.NewChar(1:ncc1, "loc") 
  setnames(dtp, c("Ind", "Pop", locc1n) )
  setcolorder(dtp, c("Ind", "Pop", locc1n))      # order the column for consistent column names
  setkey(dtp, "Ind")
  nn <- round(nrp/basel, digits = 0)
  fdata <- matrix(NA, nn, 10)
  rowse <- 0
  for (i in 1: nn){
   
    ifelse (i==1L, rowse1 <- nrp, ifelse(anTY == 1L, rowse1 <- rowse, rowse1 <- nrp-rowse))
    dtps <- copy(dtp)
    dts <- dtps[1:rowse1, ]
    setkey(dts,"Pop")
    setcolorder(dts, c("Ind", "Pop", locc1n))
    dts_refine <- DT.pop.refine(dts)
    anr <- dts_refine[, .N]
    amova_out <- numeric(9)
    if (isTRUE(anr >=2L)){
      setkey(dts_refine, "Pop")
      setcolorder(dts_refine,c("Ind", "Pop", locc1n))
      if (isTRUE(Dyt == 1L) | isTRUE(Dyt == 2L) |isTRUE(Dyt == 3L) ) { 
        amova_out<- DT.dom.hap.SSR.SNP.amova.fun(dts_refine, cs4, NCRDS4)
      }
      if (isTRUE(Dyt == 4L) )  {
        amova_out<- DT.codom.mutiple.alles.SSR.or.SNP.genotype.amova.fun(dts_refine, Dyt, cs4, NCRDS4)
      }
      if (isTRUE(Dyt == 5L))   {
        amova_out<- DT.dip.codom.2.alle.SNP.Amova.foreach.fun(dts_refine, cs4, NCRDS4)
      }
    }
    ifelse (i == 1L, ifelse(anTY == 1L, fdata[1, ] <- c(nrp, amova_out),
                            ifelse(anTY == 2L, fdata[1, ] <- c(rowse, amova_out))),
            fdata[i, ] <- c(rowse, amova_out))
    ifelse(i==1L, rowse <- basel, rowse <- rowse+basel)
  }
  ifelse(anTY==1L, colnames(fdata)<- c("Selected samples", "Samples for AMOVA ", "popN","df","SSa","SSw", "Va","Vw", "PVa","PVw"),
         colnames(fdata)<- c("Removed samples", "Samples for AMOVA", "popN","df","SSa","SSw", "Va","Vw", "PVa","PVw"))
  return(fdata)
}

#---------------------------------------------------------------------------
# labelling function for PCoA plot 
#---------------------------------------------------------------------------

DT.high.AD.select.refine.amova.plot.lable <- function(Ind.Pon, bse, Analty){
  dtt <- copy(Ind.Pon)
  setnames(dtt, c("Ind","Pop"))
  setkey(dtt, "Ind")
  ttn <- dtt[, .N]
  rse <- 0
  tm <- round(ttn/bse, digits = 0)
  selab <- matrix(NA, ttn, tm)
  for (k in 1:tm){
    ifelse(k==1L, rse1 <- ttn, ifelse(Analty == 1L, rse1 <- rse, rse1 <- ttn-rse))
    dtw <- copy(dtt)
    dtse <- dtw[1:rse1, ]
    setkey(dtse,"Pop")
    dtseN <- dtse[, .N, by = "Pop"]
    dtsef <- merge(dtseN, dtse, by = "Pop", all = TRUE, allow.cartesian=FALSE)[N>1L]
    dtsef[, c("N") := NULL]
    setcolorder(dtsef, c("Ind","Pop"))
    setkey(dtsef, "Ind")
    setkey(dtw, "Ind")
    deme <- merge(dtw, dtsef, all.x =TRUE)
    if (isTRUE(Analty == 1L)){
      deme[!(is.na(Pop.y)), Pop.y1 := "2"]
      deme[is.na(Pop.y), Pop.y1 := "1"]
    }
    if(isTRUE(Analty == 2L)){
      deme[!(is.na(Pop.y)), Pop.y1 := "1"]
      deme[is.na(Pop.y), Pop.y1 := "2"]
    }
    selab[, k] <- deme[, Pop.y1]
    ifelse(k==1L, rse <- bse, rse <- rse+bse)
  }
  dtn.lab <- data.table(dtt, selab)
  ifelse(Analty == 1L, setnames(dtn.lab, c("adInd","Pop", paste0("Selected_Label_", 1:tm, sep=""))),
         setnames(dtn.lab, c("adInd","Pop", paste0("Removed_Label_", 1:tm, sep=""))))
  setkey( dtn.lab, "adInd")
  return( dtn.lab)
  }

#######################################

DT.random.seq.select.high.AD.amova <- function(DT.adind.popr.markers, sel_seq, Dyt, anty, cs5, NCRDS5){
  
 DT.pop.refine <- function(Dtr.ind.pop.mar){   
    Dtorr <- copy(Dtr.ind.pop.mar)
    nrc1 <- length(names(Dtorr))-2
    locnrc1 <- Indchar.NewChar(1:nrc1, "loc")
    setnames(Dtorr, c("Ind","Pop",locnrc1))  
    setkey(Dtorr, "Pop")
    DtorrN <- Dtorr[, .N, by = "Pop"][N>1L]   # filter out the single sample population for AMOVA analysis
    DtorrN[, "N" := NULL]
    Dtorref <- merge(DtorrN, Dtorr, by = "Pop") 
    setcolorder(Dtorref, c("Ind","Pop",locnrc1))
    setkey(Dtorref, "Ind")
    return(Dtorref)
  }
  
  dtp <- copy(DT.adind.popr.markers)
  nrp <- dtp[, .N]
  ncc1 <- length(names(dtp))-2
  locncc1 <- Indchar.NewChar(1:ncc1, "loc")
  setnames(dtp, c("Ind", "Pop", locncc1))
  setcolorder(dtp, c("Ind", "Pop", locncc1))      # order the column for consistent column names
  setkey(dtp, "Ind")
  ifelse(anty==1L, rowse <- sel_seq, rowse <- nrp-sel_seq)
  rowse <- setdiff(rowse, 0)
  nns <- length(rowse)
  fdata <- matrix(NA, nns+1, 10)
  
  for (i in 0:nns){
    ifelse (i==0L, rowse1 <- nrp, rowse1 <- rowse[i])
    dtps <- copy(dtp)
    dts <- dtps[1:rowse1,]
    setkey(dts,"Pop")
    setcolorder(dts,c("Ind", "Pop", locncc1))
    dts_refine <-DT.pop.refine(dts)
    anr <- dts_refine[, .N]
    amova_out <- numeric(9)
    if(isTRUE(anr >= 2)){
      setkey(dts_refine, "Pop")
      setcolorder(dts_refine,c("Ind", "Pop", locncc1))
      
      if (isTRUE(Dyt == 1L) | isTRUE(Dyt == 2L) |isTRUE(Dyt == 3L) ) { 
        amova_out<- DT.dom.hap.SSR.SNP.amova.fun(dts_refine, cs5, NCRDS5 ) 
      }
      if (isTRUE(Dyt == 4L))  {
        amova_out<- DT.codom.mutiple.alles.SSR.or.SNP.genotype.amova.fun(dts_refine, Dyt, cs5, NCRDS5 )
      }
      if (isTRUE(Dyt == 5L))  {
        amova_out<- DT.dip.codom.2.alle.SNP.Amova.foreach.fun(dts_refine,Dyt,cs5, NCRDS5 )
      }
      
    }
    ifelse (i == 0L, ifelse(anty == 1L, fdata[1, ] <- c(nrp, amova_out),
                            ifelse(anty == 2L, fdata[1, ] <- c(nrp-rowse1, amova_out))),
                     ifelse(anty == 1L, fdata[i+1, ] <- c(rowse1, amova_out),
                           ifelse(anty == 2L, fdata[i+1, ] <- c(nrp-rowse1, amova_out))))
  }
  ifelse(anty == 1L, colnames(fdata)<- c("Selected samples", "Samples for AMOVA ", "popN","df","SSa","SSw", "Va","Vw", "PVa","PVw"),
                     colnames(fdata)<- c("Removed samples", "Samples for AMOVA", "popN","df","SSa","SSw", "Va","Vw", "PVa","PVw"))
  return(fdata)
}

##########################################

DT.random.seq.select.high.AD.refine.amova.plot.label <- function(Ind.Pona, bs_seq, nalty){
  dtt <- copy(Ind.Pona)
  setnames(dtt, c("Ind","Pop"))
  setkey(dtt, "Ind")
  ttn <- dtt[, .N]
  ifelse(nalty == 1L, rsel <- bs_seq, rsel <- ttn-bs_seq)
  rsel <- setdiff(rsel,0)
  tm <- length(rsel)
  rse <- 0
  selab <- matrix(NA, ttn, tm+1)
  for (k in 0:tm){
    ifelse(k == 0L, rse <- ttn, rse <- rsel[k])
    dtw <- copy(dtt)
    dtse <- dtw[1:rse, ]
    setkey(dtse,"Pop")
    dtseN <- dtse[, .N, by = "Pop"]
    dtsef <- merge(dtseN, dtse, by = "Pop", all = TRUE, allow.cartesian=FALSE)[N>1L]
    dtsef[, "N" := NULL]
    setcolorder(dtsef, c("Ind","Pop"))
    setkey(dtsef, "Ind")
    setkey(dtw, "Ind")
    deme <- merge(dtw, dtsef, all.x =TRUE)
    if (isTRUE(nalty == 1L)){
      deme[!(is.na(Pop.y)), Pop.y1 := "2"]
      deme[is.na(Pop.y), Pop.y1 := "1"]
    }
    if(isTRUE(nalty == 2L)){
      deme[!(is.na(Pop.y)), Pop.y1 := "1"]
      deme[is.na(Pop.y), Pop.y1 := "2"]
    }
    selab[, k+1]<-deme[, Pop.y1]
  }
  
  dtn.lab <- data.table(dtt, selab)
  ifelse(nalty==1L, setnames(dtn.lab, c("adInd","Pop", paste0("Selected_Label_", 0:tm, sep=""))),
                    setnames(dtn.lab, c("adInd","Pop", paste0("Removed_Label_", 0:tm, sep=""))))
  setkey(dtn.lab, "adInd")
  return(dtn.lab)
}

############################

selected.PCoA.plot <- function(adPCoAvalues, kipe2, IDpop3, p12, p22, anTy2, marTy2, Pro){
  
  PCoAv <- copy(adPCoAvalues)
  setnames(PCoAv, c("PCoA1","PCoA2", "PCoA3", "PCoA4"))
  PCoAvNames <- names(PCoAv)
  PCoA1v <- PCoAvNames[p12]
  PCoA2v <- PCoAvNames[p22]
  PCoA12 <- unlist(PCoAv[, ..PCoA1v])
  PCoA22 <- unlist(PCoAv[, ..PCoA2v])
  xld12 <- c(min(PCoA12)-0.001, 1.2*max(PCoA12))
  yld12 <- c(min(PCoA22)-0.001, 1.2*max(PCoA22))
  
  IDpob2 <- copy(IDpop3)
  ppn <- length(names(IDpob2))-1
  setnames(IDpob2, c("adInd",paste("pop", 1:ppn, sep = "")))
  setkey(IDpob2,"adInd")
  IDpobNames <- names(IDpob2)
  
  for (i in 1:ppn){
    j <- i+1
    popsel <- IDpobNames[j]
    popl <-IDpob2[, ..popsel] 
    dat2 <- data.table(PCoA12, PCoA22, popl )
    setnames(dat2, c("ppc1","ppc2", "popl"))
    setkey(dat2, popl)
    nrd <- dat2[, .N]
    popn <- dat2[, .N, by = popl]
    mm <- popn[, N]
    PCs11 <- dat2[, ppc1]
    PCs22 <- dat2[, ppc2]
    selt <- as.character()
    cols <- as.character()
    mycol <- as.character()  
    if(isTRUE(length(mm)==2L)){
      selt <- c("Unselected", "Selected")
      cols <- c("grey", "black")
      mycol <- rep(c("grey","black"), c(mm[1], mm[2]))
    }
    if(anTy2== 1L) {
      pdf(file=paste(marTy2,"_", anTy2,"_", Pro,"_", j, "_","selected_", mm[2], "_among_", nrd, "_samples_",p12,"_",p22,"_","PCoA", ".pdf", sep= "" ),
          width=7, height=7, paper="special", fonts="Helvetica")
      plot(PCs11, PCs22, pch=20, col=mycol, cex = 1.0, xlab = paste("PCoA1"," ", "(",round(kipe2[p12],2), "%",")", sep=" "),
           ylab = paste("PCoA2"," ","(", round(kipe2[p22],2),"%", ")",sep=" "), xlim = xld12, ylim = yld12)
      abline(h = 0, v = 0, col = "gray60", lty=3)
      title(paste("A PCoA plot of ",mm[2], " Selected ","(", "black", ")"," vs ","original samples",  sep=" "))
      legend("topleft", legend=paste(selt), col = cols, pch=16, bty="n",ncol=1, cex=1.0, pt.cex=0.9)
      dev.off()
    }
    if(isTRUE(anTy2== 2L)) {
      pdf(file=paste(marTy2,"_", anTy2,"_", Pro,"_", j, "_", "removed_", mm[2], "_among_", nrd, "_samples_",p12,"_",p22,"_","PCoA", ".pdf", sep= "" ),
          width=7, height=7, paper="special", fonts="Helvetica")
      plot(PCs11, PCs22, pch=20, col=mycol, cex = 1.0, xlab = paste(paste("PCoA", p12, sep="")," ","(", round(kipe2[p12],2), "%", ")",sep=" "),
           ylab = paste(paste("PCoA", p22, sep="")," ","(" ,round(kipe2[p22],2), "%",")", sep=" "), xlim = xld12, ylim = yld12)
      abline(h = 0, v = 0, col = "gray60", lty=3)
      title(paste("A PCoA plot of ",mm[2], " Selected ","(", "black", ")"," vs ","original samples",  sep=" "))
      legend("topleft", legend=paste(selt), col = cols, pch=16, bty="n",ncol=1, cex=1.0, pt.cex=0.9)
      dev.off()
    }
  }
}


##################################### 
# Main program
#####################################

#----------------------------------------------------------------------------------------------------------------
#  1. Raw Inpop_markers used for Ind and Pop format 
#----------------------------------------------------------------------------------------------------------------
Fmark_trans <- Raw.trans.Ind.Pop.rename.Mark.Format.NA.replace(Inpop_markers, Miss_Data, Marker_Type)    
setkey(Fmark_trans, "Ind")
Fmark_trans1 <- copy(Fmark_trans)

Raw_Indx <-Fmark_trans1[, .(Ind)] 
rIndmar <- Fmark_trans1[, c("Pop","IndR","PopR"):= NULL]           
rm(Fmark_trans1)

Fu_dissim_mat <- DT.dom.hap.codom.AD.mat(rIndmar, ccs, NCreads) 
rm (rIndmar)

Fu_AD <- apply(Fu_dissim_mat, 1, mean, na.rm = TRUE)
Fu_AD_plot <- AD.count.plot(Fu_AD, Marker_Type, Analysis_Type)       # plot AD distribution
Fu_AD_Ind2_Pop2_Marker_DT <- data.table(Fu_AD, Fmark_trans)          # combine the AD with ind,pop and marker DT
rm(Fmark_trans)

#----------------------------------------------------------------------------------------------------------------
# 2. Calculate PCoA value
#----------------------------------------------------------------------------------------------------------------

Fu_dissim_mat[is.na(Fu_dissim_mat)] <- 0
ann2 <- nrow(Fu_dissim_mat)

Ad_dist <- as.dist(Fu_dissim_mat, upper = FALSE)
Ad_dist <- sqrt(Ad_dist)
res <- cmdscale(Ad_dist, k = (NROW(Fu_dissim_mat) - 1), eig = TRUE, add = TRUE)
eig2 <- eigenvals(res)
kipe <- 100 * eig2/sum(eig2)               # PCoA eigenvalues proportion

PCoA_value_Raw <- res$points
PCoA_vector <- data.table(Raw_Indx, Fu_AD, PCoA_value_Raw[, 1:4])                         # combine the raw Indx, AD with PCoA values
setnames(PCoA_vector, c("Raw_Indx","AD","PCoA1","PCoA2","PCoA3","PCoA4"))
setkey(PCoA_vector, "Raw_Indx") 
PCoA_vector <- PCoA_vector[ , lapply(.SD, as.numeric), .SDcols= c("AD","PCoA1","PCoA2","PCoA3","PCoA4"), by = Raw_Indx]
Fu_ad_PCoA_vector0 <- PCoA_vector[order(-AD)]                                             # order -AD from high to low to reorder the PCoA values

#----------------------------------------------------------------------------------------------------------------
# 3. Generate AD descending data file 
#----------------------------------------------------------------------------------------------------------------

des_Fu_AD_Ind3_Pop3_Marker_DT <- DT.AD.desced.index.random(Fu_AD_Ind2_Pop2_Marker_DT)     # output has 6 columns for marker information
rm(Fu_AD_Ind2_Pop2_Marker_DT)
desNames <- names(des_Fu_AD_Ind3_Pop3_Marker_DT)
setnames(des_Fu_AD_Ind3_Pop3_Marker_DT, c("adInd", "AD", "Ind","Pop","IndR","PopR",desNames[-c(1:6)]))

Fu_adInd <- des_Fu_AD_Ind3_Pop3_Marker_DT[, adInd]
Fu_Pop <- des_Fu_AD_Ind3_Pop3_Marker_DT[, Pop]
Fu_IndR <- des_Fu_AD_Ind3_Pop3_Marker_DT[, IndR]
Fu_PopR <- des_Fu_AD_Ind3_Pop3_Marker_DT[, PopR]

Fu_ad_PCoA_vector <- data.table(Fu_adInd, Fu_Pop, Fu_IndR, Fu_PopR, Fu_ad_PCoA_vector0)

setnames(Fu_ad_PCoA_vector, c("adInd","Pop","IndR","PopR","Ind","AD","PCoA1","PCoA2","PCoA3","PCoA4"))
write.csv(Fu_ad_PCoA_vector, file =paste(Marker_Type,"_", Analysis_Type,"_",stepwise_selection,"_",ann2,
                                  "_Fu_AD_order","_first_4_PCoA_vectors_results",".csv",sep=""))

Fu_adInd_Pop_Markers <- des_Fu_AD_Ind3_Pop3_Marker_DT[, c("AD","Ind","IndR","PopR"):= NULL, ]
rm(des_Fu_AD_Ind3_Pop3_Marker_DT)
locnamc <- Indchar.NewChar(1:Loci_Ns, "loc")
setnames(Fu_adInd_Pop_Markers,c("Ind","Pop",locnamc))   # for AMOVA analysis
setkey(Fu_adInd_Pop_Markers, "Ind")                                              # adInd keyed 

#----------------------------------------------------------------------------------------------------------------
# 4. select high AD samples to assess changes in genetic structure by AMOVA 
#----------------------------------------------------------------------------------------------------------------

Fu_adInd_Pop_Markers1 <- copy(Fu_adInd_Pop_Markers)
Ind_Pop1<- c("Ind","Pop")

setnames(Fu_adInd_Pop_Markers1, c(Ind_Pop1, locnamc))
Fu_adInd_Pop <- Fu_adInd_Pop_Markers1[, ..Ind_Pop1]
marDTc <- Fu_adInd_Pop_Markers1[, ..locnamc]
Nss <- Fu_adInd_Pop[, .N]

Fu_adInd_Pop_Markers2 <- data.table(Fu_adInd_Pop, marDTc)
setnames(Fu_adInd_Pop_Markers2, c("Ind", "Pop", locnamc))
  
  if (isTRUE(Marker_Type == 5L)){
  Ind2Locme1239M <- Codom.SNP.alle.convert.ATCG.to.number1239.fun (Fu_adInd_Pop_Markers2, ccs, NCreads)
  fwrite(Ind2Locme1239M, file="MarkType_5_two_alles_convert_Ind2Locme1239M.txt")
  Ind2Lcnames <- names(Ind2Locme1239M)
  IndsPop1239Mat3 <- Ind2Locme1239M[-1]
  setnames(IndsPop1239Mat3, Ind2Lcnames)
  Fu_adInd_Pop_Markers2 <- copy(IndsPop1239Mat3)
}

Fu_Ad_selected_amova_level_labels_stepwise <- data.table()    # output labels for plot
Fu_Ad_selected_amova_level_labels_specific <- data.table()

if (isTRUE(!is.na(bas1)) & isTRUE(bas1 > 0L)) {
  bas2 <- bas1 
  Fu_Ad_selected_amova_df <- DT.simu.high.AD.stepwise.selected.amova(Fu_adInd_Pop_Markers2, bas2, Marker_Type, Analysis_Type, ccs, NCreads)  #  704  #809 812
  Fu_Ad_selected_amova_level_labels_stepwise <- DT.high.AD.select.refine.amova.plot.lable(Fu_adInd_Pop, bas2, Analysis_Type)

  if(isTRUE(Analysis_Type== 1L)){
     write.csv(Fu_Ad_selected_amova_df, file=paste(Marker_Type, "_", Analysis_Type, "_", stepwise_selection, "_", Nss,
    	 "_Fu_AD_order_stepwise_selected_amova_results",".csv",sep=""))
   }

  if(isTRUE(Analysis_Type== 2L)){
     write.csv(Fu_Ad_selected_amova_df, file=paste(Marker_Type, "_", Analysis_Type, "_", stepwise_selection, "_", Nss,
    	 "_Fu_AD_order_stepwise_removed_amova_results",".csv",sep=""))
   }
}

if (isTRUE(!is.na(seqlabel)) & isTRUE(seqlabel > 0L)){
  bas_seq <- seqlabel 
  Fu_Ad_selected_amova_op <-DT.random.seq.select.high.AD.amova(Fu_adInd_Pop_Markers2, bas_seq, Marker_Type, Analysis_Type,ccs,NCreads)
  Fu_Ad_selected_amova_level_labels_specific <- DT.random.seq.select.high.AD.refine.amova.plot.label(Fu_adInd_Pop, bas_seq, Analysis_Type)
  
  if(isTRUE(Analysis_Type== 1)){
    write.csv(Fu_Ad_selected_amova_op, file=paste(Marker_Type,"_", Analysis_Type,"_", Nss,
	           "_Fu_AD_order_specific_selected_amova_results",".csv",sep=""))
   }
  
  if(isTRUE(Analysis_Type== 2)){
    write.csv(Fu_Ad_selected_amova_op, file=paste(Marker_Type,"_", Analysis_Type,"_", Nss,
	    "_Fu_AD_order_specific_removed_amova_results",".csv",sep=""))
   }
 }
rm(Fu_adInd_Pop_Markers2)

#----------------------------------------------------------------------------------------------------------------
# 5.  PCoA plot displays 
#----------------------------------------------------------------------------------------------------------------

PCAMat <- copy(Fu_ad_PCoA_vector)  
setnames(PCAMat, c("adInd","Pop","IndR","PopR","Ind","AD","PCoA1","PCoA2","PCoA3","PCoA4"))
PCA4valuesName <- c("PCoA1","PCoA2","PCoA3","PCoA4")
Fu_ad_PCoAV <- PCAMat[, ..PCA4valuesName]

if (isTRUE(!is.na(bas1)) & isTRUE(bas1 > 0L)){
  StepwiseLables <- names(Fu_Ad_selected_amova_level_labels_stepwise)
  StepwiseSelectLables <- StepwiseLables[-c(2:3)]
  Fu_lables <- Fu_Ad_selected_amova_level_labels_stepwise[, ..StepwiseSelectLables]
  select_plot <- selected.PCoA.plot(Fu_ad_PCoAV, kipe, Fu_lables, 1, 2, Analysis_Type, Marker_Type, stepwise_selection)
}

if (isTRUE(!is.na(seqlabel)) & isTRUE(seqlabel > 0L)){
  setnames(Fu_Ad_selected_amova_level_labels_specific, c("adInd", "pop", "Selected_Label_0", "Selected_Label_1"))
  Fu_uselablesName <- c("adInd", "Selected_Label_1")
  Fu_lables <- Fu_Ad_selected_amova_level_labels_specific[, ..Fu_uselablesName]
  select_plot <- selected.PCoA.plot(Fu_ad_PCoAV, kipe, Fu_lables, 1, 2, Analysis_Type, Marker_Type, specific_selection)
  }

#----------------------------------------------------------------------------------------------------------------
# 6. Save and move outputs to the results subdirectory
#----------------------------------------------------------------------------------------------------------------

flist1 <- list.files(path = path1, pattern = "*average_dissimilarity*", all.files = FALSE,  full.names = TRUE,
                     recursive = FALSE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE)
ifelse(Analysis_Type==1L, flist2 <- list.files(path = path1, pattern = "*selected_amova*", all.files = FALSE,  full.names = TRUE,
                                                recursive = FALSE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE), 
                        flist2 <- list.files(path = path1, pattern = "*removed_amova*", all.files = FALSE,  full.names = TRUE,
                                                recursive = FALSE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE)) 
flist3 <- list.files(path = path1, pattern = "*PCoA.pd*", all.files = FALSE,  full.names = TRUE,
                     recursive = FALSE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE)
flist4 <- list.files(path = path1, pattern = "*PCoA_vectors*", all.files = FALSE,  full.names = TRUE,
                     recursive = FALSE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE)
flist5 <- list.files(path = path1, pattern = "^MarkType_5_two_alles*", all.files = FALSE,  full.names = TRUE,
                     recursive = FALSE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE)
file.copy(from = c(flist1, flist2,  flist3, flist4), to = workpath[work4], overwrite = TRUE, copy.mode = TRUE)
file.remove(c(flist1, flist2, flist3, flist4, flist5))

#----------------------------------------------------------------------------------------------------------------
# 7. exit
#----------------------------------------------------------------------------------------------------------------

quit()
 

