#########################################################################################
#                                                          							              	
#  This R script AveDissR.r was developed in 2017 for assessing genetic distinctness	  
#  and genetic redundancy	 															                                
#                                                          								            
#  Please read "Using AveDissR.pdf" before using this script.      			  
#                                                          				 
#  For more information please contact:                  				 
#                                                      				    	 
#     Dr. Yong-Bi Fu                                  				     	 
#     Agriculture & Agri-Food Canada                 				      	 
#     107 Science Place                            				         
#     Saskatoon SK Canada S7N 0X2                  				        
#     tel: +1-306-385-9298                         				        
#     email: Yong-Bi.Fu@canada.ca    							 
#											 
#  Disclaimer: The R script AveDissR.r and related files are published as it is and	 
#    distributed free of charge and without warranty of any kind, and no guarantee	 
#    is provided as to the suitability or functioning of this software or to the	 
#    results obtained from its use							 
#                                                     				     	 
#########################################################################################
#
# Note: after version v.1.12.+ in data.table, the reshape2 package is not required to be loaded before loading data.table.
# The current vagen package needs loading the two dependent packages of permute and lattice before load vagen.
# The current doParallel package needs  to load two dependent packages of iterators and parallel before load doParallel
# Please install the seven R packages before using this AveDissR.r script, updated at 2021-2-26
#---------------------------------------------------------------------------------------------
# install.packages("permute", dependencies = TRUE)
# install.packages("lattice", dependencies = TRUE)
# install.packages("data.table", dependencies = TRUE)
# install.packages("vegan",dependencies = TRUE)
# install.packages("foreach",dependencies = TRUE)
# install.packages("iterators",dependencies = TRUE)
# install.packages("doParallel",dependencies = TRUE)

# This new version of AveDissR_v2.r was tested on window R4.03 and linux Redhat R4.03 on March 27, 2021
# Users can set up the number of cores to be used for parallel computing on line 60.
# The rule of thumb is to use less than half of the cores available.
#########################################################################################

#---------------------
# library packages
#---------------------
library(data.table)
library(permute)
library(lattice)
library(vegan)
library(foreach)
library(iterators)
library(parallel)
library(doParallel)

rm(list=ls())

#----------------------------------------------------------------------------------------------------------------------------------------------------------------------
# 1.1  Setting the running environment for parallel run parameters:   ccs <- makeCluster(10), means there are 10 cores for parallel run in foreach function
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------

NCreads <- 10          # users can set up the number of cores used in the program; the default has 10 cores.

ccs <- makeCluster(NCreads)   
registerDoParallel(ccs)       
setDTthreads(NCreads)     

#-----------------------------------------------------------------------
# Get working directory and file
#----------------------------------------------------------------------- 
path1 <- getwd()
workpath <- list.dirs(path = path1, full.names = TRUE, recursive = TRUE)
cfolders <- basename(workpath)
work3 <- grep("^inputdata$", cfolders, ignore.case = TRUE, value=FALSE)
work4 <- grep("^results$", cfolders, ignore.case = TRUE, value= FALSE)
 
Input_files <- list.files(path = workpath[work3], pattern = NULL, all.files = TRUE,  full.names = TRUE,
                              recursive = TRUE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE)

if (isTRUE(length(Input_files) != 2 )){ 
  stop(print( c("Errors: Input data must be two files, one for parameters setting and the other for marker data input. 
                The parameter setting file should be named with  'parameters_setting'prefixed file name." )))
}

filebase <- basename(Input_files)
PA_index <- grep("parameters_setting", filebase, ignore.case = TRUE, value=FALSE)

#--------------------------------------------------------------------------------------------
# Read the marker data and parameters files into working directory from the inputdata folder
#--------------------------------------------------------------------------------------------
Parameters <- c()
Inpop_markers <- data.table()
bas1 <- c()
seqlabel <- c()

if (isTRUE(length(Input_files) == 2 )){ 
  Parameters_set <- read.csv(file = Input_files[PA_index], header = TRUE, strip.white = TRUE, na.strings = c("NA",NULL), sep=",", fill =TRUE)  # input parameter_setting.csv as a data.frame
  Parameters_set <- data.frame(Parameters_set)
  Parameters <- Parameters_set[1:8, 2]
  
  Inpop_markers <- fread(input = Input_files[-PA_index], header = FALSE, strip.white = TRUE, na.strings = c("NA",NULL),sep=",",
                            stringsAsFactors=FALSE, fill =TRUE)  # input data as a data.table for fast read into memory
 }

#-----------------------------------------------------------------------------------------------
# Analysis types: 1 genetic distinctness, 1; genetic redundancy, 2 
# Five marker data types: dominant marker, 1; haploid SSR or binary marker, 2; haploid SNP, 3; 
#                         diploid SSR with a separator ":" between alleles, 4; and
#                         diploid SNP with no separator between alleles,5.
#-----------------------------------------------------------------------------------------------
Analysis_Type <- as.numeric(Parameters[1])   # 1 Genetic distinctness, 2 genetic redundancy  
Marker_Type <- as.numeric(Parameters[2])
Ind_N <-  as.numeric(Parameters[3])
Pop_N <-  as.numeric(Parameters[4])
Loci_N <- as.numeric(Parameters[5])
Miss_Data <- as.character(Parameters[6], na.rm = TRUE) 
stepwise_selection <- as.numeric(Parameters[7], na.rm = TRUE)
specific_selection <- as.numeric(Parameters[8], na.rm = TRUE)
   
Ind_Ns <-  length(unique(unlist(Inpop_markers[1, -1, with = FALSE])))
Pop_Ns <-  length(unique(unlist(Inpop_markers[2, -1, with = FALSE])))
Loci_Ns <- Inpop_markers[, .N]-2
 
# Checking the input data consistent with the parameters setting
if(isTRUE(Ind_N != Ind_Ns) | isTRUE(Pop_N != Pop_Ns) | isTRUE(Loci_N != Loci_Ns)) { 
     stop(print(c("Errors: the parameters setting is not consistent with the marker data format.
                   Please check the inputdata folder to format the two inputdata files")))
}

ifelse (isTRUE(is.na(stepwise_selection)),  bas1 <- NA,
         ifelse(stepwise_selection > 0L & stepwise_selection < 1L, bas1 <- round(Ind_Ns * stepwise_selection, 0), 
                                         stop(print(c("Errors: The stepwise selection value should be set between 0 and 1,
                                                      please check the stepwise_selection parameter setting.")))))
 
ifelse (isTRUE(is.na(specific_selection)), seqlabel<- NA,
         ifelse(specific_selection > 0L & specific_selection < 1L, seqlabel <- round(Ind_Ns * specific_selection, 0),
                                        stop(print(c("Errors: The specific selection value should be set between 0 and 1, 
                                                     please check the specific_selection parameter setting.")))))
													 
#-----------------------------------------
# get indexed individual, population name
#-----------------------------------------

Indline.rename <- function(Mtchar, rechar){   # Mtchar is a set of characters, rechar is a character for new order beginning)
  ns <- length(Mtchar)
  nc <- nchar(ns)
  chtc <- paste(rechar,sapply(mapply(rep,nc-nchar(1:ns),MoreArgs=list(x=0)),paste,collapse=''), 1:ns,sep="")
  return(chtc)
}

Popline.rename <- function(pops, ch1){
  IDpo <- as.character(pops)
  ch1 <- as.character(ch1)
  Upo <- unique(IDpo) 
  po_N <-length(Upo)
  for (j in 1:po_N){
    ifelse(j==1,  
           IDpo[IDpo %in% Upo[j]] <- paste(ch1, "_000000", sep = ""),
           ifelse(1 < j & j <= 10,
                  IDpo[IDpo %in% Upo[j]] <- paste(ch1, "_00000", j-1, sep = ""),
                  ifelse(10 < j & j <= 100, 
                         IDpo[IDpo %in% Upo[j]]<-paste0(ch1, "_0000",j-1, sep = ""),
                         ifelse(100 < j & j <= 1000, 
                                IDpo[IDpo %in% Upo[j]] <-paste0(ch1, "_000",j-1, sep = ""),
                                ifelse(1000 < j & j <= 10000, 
                                       IDpo[IDpo %in% Upo[j]] <-paste0(ch1, "_00",j-1, sep = ""),
                                       ifelse(10000 < j & j <= 100000, 
                                              IDpo[IDpo %in% Upo[j]] <-paste0(ch1, "_0",j-1, sep = ""),
                                              IDpo[IDpo %in% Upo[j]] <-paste0(ch1, "_", j-1, sep = "")))))))
  }
  return(IDpo)
}

#---------------------------------------------------------------------------
# check marker matrix with missing data and replace with NA (missing data)
#---------------------------------------------------------------------------
Miss.NA.replace <- function(DTo1, MisD, Datt) {  # input data is just for markers, no pop label, Datt is data type
  DTo <- copy(DTo1)
  if(!is.na(MisD)){
    for (j in names(DTo)){
      set(DTo,which(DTo[[j]]%in% MisD), j, NA_character_ )
    }
  }
  return(DTo)
}

Format.marker <- function(raw.marker.DT, missd, DaTy){
  marker <-copy(raw.marker.DT)
  missd <- as.character(missd, na.rm = TRUE)
  marker <- marker[, lapply(.SD,   function(x) as.character(x, na.rm = TRUE))]  # Dominant marker data are coded for 0,1,9(or NA), all are convert into characters
  marker_NA <- Miss.NA.replace(marker, missd, DaTy)                             # haploid, diploid SSR or SNPs are characters
  return(marker_NA)
}

dip.SSR.SNP.alles.min.max.reorder <- function(DT.Ind.Pop.MK, MarTy){
  DTinPM <- copy(DT.Ind.Pop.MK)
  IndP <- DTinPM[,1:4, with=FALSE]
  narm <- names(DTinPM)
  dc <- length(narm)
  dloc <- dc-4
  Indk <- IndP[,1, with=FALSE]
  mKdata <- DTinPM[,c(narm[1:4]):= NULL] 
  mKdata <- data.table(Indk,mKdata)
  locAA <- Indline.rename(1:dloc, "loc")
  setnames( mKdata, c("Ind",locAA ))
  mKdata_melt <- melt(mKdata, measure= list(locAA), value.name=c("alles"))
  ifelse(MarTy==4L, mKdata_melt[, c("p1","p2"):= tstrsplit(alles, ":", fixed=TRUE)],
                    mKdata_melt[, c("p1","p2"):= tstrsplit(alles, "", fixed=TRUE)])
  ifelse(MarTy==4L, mKdata_melt[, c("p1p2"):= paste0(min(p1,p2),":" , max(p1,p2),sep=""), by=c("Ind", "variable")],
                    mKdata_melt[, c("p1p2"):= paste0(min(p1,p2) , max(p1,p2),sep=""), by=c("Ind", "variable")])
  ifelse(MarTy==4L, mKdata_melt[p1p2%in%c("NA:NA"), c("p1p2"):= NA_character_], mKdata_melt[p1p2%in%c("NANA"), c("p1p2"):= NA_character_])
  mKdata_melt[, c("alles"):= p1p2]
  mKdata_melt[, c("p1", "p2", "p1p2"):= NULL]
  mKdata_dcast <- dcast(mKdata_melt, Ind ~ variable, value.var = c("alles"), value.name= c("loc"))
  # setDT(mKdata_dcast) # if library(reshape2), you need to add the syntax, as dcast in reshape2 get a data.frame,not a data.table. However, load data.table only would  get a data.table.
  mKdata_dcast[, c("Ind"):= NULL]
  mKdata_dcast <- data.table(IndP,mKdata_dcast)
  return(mKdata_dcast)
}

#---------------------------------------------------------------------------
# Get formatted individual,pop, marker NA to replace matrix data.table (DT)
#---------------------------------------------------------------------------
Raw.Ind.Pop.rename.Mark.Format.NA.replace <- function(Raw.DT.IPline.Mar, mIss, Mty){
  Raw.DT.IPM <- copy(Raw.DT.IPline.Mar)
  Indline <- unlist(Raw.DT.IPM[1, ])
  IndRename <- Indline.rename(Indline, "N")
  Indline2 <- rbind(IndRename,Indline)
  IndRemat2 <- data.table(Indline2)
  Popline <- unlist(Raw.DT.IPM[2, ])
  PopRename <- Popline.rename(Popline, "P")
  Popline2 <- rbind(PopRename,Popline )
  PopRemat2 <- data.table(Popline2) 
  LMarkers <- Raw.DT.IPM[-c(1:2), ]
  Lociname <-LMarkers[, 1, with = FALSE]
  MarkerS <- LMarkers[, -1, with = FALSE]
  FMarkerS <- Format.marker(MarkerS, mIss, Mty)
  MarkerS_NA <- Miss.NA.replace(FMarkerS, mIss, Mty) 
  LmarkerSNA <- data.table(Lociname, MarkerS_NA)
  nlist <- list( IndRemat2, PopRemat2, LmarkerSNA)
  ReFmarkers <- rbindlist(nlist,use.names=FALSE, fill=FALSE) # update the rbindlist() in use.names=FALSE to bind by position.2019-8-22
  hder<- as.character(unlist(ReFmarkers[1, ]))
  setnames(ReFmarkers, hder)
  Atras <- transpose(ReFmarkers, fill=NA, ignore.empty=FALSE)    # transpose the marker data for use
  col_name<-as.character(unlist(Atras[1, ])) 
  setnames(Atras, col_name)
  IPM_trans <- Atras[-1, ]
  lnc <- length(names(IPM_trans))-4
  locnm <-Indline.rename(1:lnc, "loc")
  setnames(IPM_trans, c("Ind","IndR","Pop", "PopR",locnm))
  setcolorder(IPM_trans, c("Ind","Pop","IndR", "PopR",locnm))
  if(isTRUE(Mty==4L) | isTRUE(Mty == 5L)) {
    IPM_trans <- dip.SSR.SNP.alles.min.max.reorder(IPM_trans, Mty)
  }
  return(IPM_trans)                                   # output is a transposed DT with 4 columns marker labels
}

#------------------------------------------------------------------------------------------
# To assess the changes in genetic structure, we perform AMOVA and show PCoA plot 
# Analysis type, 1: select a certain number of distinct accessions from a collection
# Analysis type, 2: remove a certain number of redundant accessions from a collection   
#------------------------------------------------------------------------------------------
headlind.mat.transpose <- function(Ind.Mark){
  DTM1 <- copy(Ind.Mark)
  hedf <- names(DTM1)
  h1 <- length(hedf)-1
  locna <-Indline.rename(1:h1, "loc") 
  setnames(DTM1, c("Ind", locna))
  setkey(DTM1, "Ind")
  hedA <- Indline.rename(hedf,"A")
  hedAT <- rbind(hedA, hedf)
  hedAT<- data.table(hedAT)
  hdmarl <- list( hedAT, DTM1)
  hIDmar <- rbindlist(hdmarl,use.names=FALSE, fill=FALSE) # update the rbindlist() in use.names=FALSE to bind by position. 2019-8-22
  locIndtrs <- transpose(hIDmar, fill=NA, ignore.empty=FALSE)
  locIndtrs[, c("V1"):= NULL]
  setnames(locIndtrs, as.character(locIndtrs[1, ]))
  locIndtrs <- locIndtrs[-1, ]
  return(locIndtrs)
}

#---------------------------------------------------------------------
# Dissimilarity matrix calculation; input DT includes ind and markers 
#---------------------------------------------------------------------

DT.dom.hap.codom.AD.mat <- function(DT.ind.domar.mat, cs1,NCRDS1){ 
  cs01 <- makeCluster(cs1)  
  registerDoParallel(cs01)  
  setDTthreads(NCRDS1)
  on.exit(stopCluster(cs01))
  DTtrans <- copy(DT.ind.domar.mat)
  locIndts <- headlind.mat.transpose(DTtrans)
  marInd <-locIndts[, -1, with=FALSE]  
  nid <- length(names(marInd))
  nr <-marInd[,.N]
  ADdis <- matrix(NA, nid, nid)
  B2 <- foreach( k = 1:nid, .combine= rbind, .packages=c('foreach','doParallel', 'data.table')) %dopar% {
    A1 <- marInd[,k, with=FALSE]
    a1 <- unlist(A1)
    A2 <- marInd[,-k, with=FALSE]
    hrm <- names(A2)
    mrc <- paste("d",hrm[-1],sep="" )
    A11 <- data.table()
    ifelse(length(hrm)==1L, A11 <- A1,
           A11 <- A1[, c(mrc):= a1])
    setDF(A11)
    setDF(A2)
    A112_ueq <- matrix(as.numeric(A11!=A2), nr, nid-1)
    A112_eq <- matrix(as.numeric(A11==A2), nr, nid-1)
    uneq <- apply(A112_ueq, 2, sum, na.rm =TRUE)
    eq <- apply(A112_eq, 2, sum,na.rm =TRUE )
    B1 <- uneq/(uneq+eq)
  }
  nn1 <- ncol(B2)
  for(k in 1: (nn1+1)){
    ifelse(k == 1, ADdis[k, ]<- c(NA, B2[k,]), 
           ifelse ( 1 < k & k < nid, ADdis[k, ] <- c(B2[k, 1:(k-1)], NA, B2[k, k:nn1]),
                    ADdis [k, ] <-c(B2[k, ], NA)))    
  }
  return(ADdis)
}

#----------------------------------------------------
# Order AD descending from high to low, and index the new Ind with new names from AD001-AD00n
# ---------------------------------------------------
 DT.AD.desced.index.random <- function(AD.ind.pop.indr.popr.marker){     #  input data is a DT with AD, ind, pop and marker
    dt.ad <- copy(AD.ind.pop.indr.popr.marker)
    mcc <- length(names(dt.ad))-5
    locnn <-Indline.rename(1:mcc,"loc") 
    setnames(dt.ad, c("AD","Ind","Pop","IndR","PopR",locnn))
    desdt.AD <- dt.ad[order(-AD)]
    mn <- desdt.AD[,.N]
    adInd <-Indline.rename(1:mn, "AD")
    adInd.Popr.des.AD.dt <- data.table(adInd,desdt.AD)
    setcolorder(adInd.Popr.des.AD.dt, c("adInd","AD","Ind","Pop","IndR","PopR",locnn))
    return(adInd.Popr.des.AD.dt)    #  Output has 6 columns for new indexed adInd and random popr, used for further analysis 
}
 
AD.count.plot <- function(AD.value, dtyy,analy){
   value0 <- as.numeric(AD.value)
   my_hist <- hist(value0, breaks = 20, plot = F, right= TRUE)
   my_breaks <- my_hist$breaks
   my_counts <- my_hist$counts
   myint <- my_breaks[2]-my_breaks[1]
   myn <- length(my_breaks)
   myx0 <- my_breaks[1] - myint
   myxn <- my_breaks[myn] + myint
   mybreaks2 <- c(myx0, my_breaks)
   mycout2 <- c(0, my_counts, 0)
   valcount <- data.frame(mybreaks2,mycout2)
   nbreak <- nrow(valcount)
   colnames(valcount) <- c("AD","count")
   pdf(file=paste(dtyy,"_",analy,"_",nbreak,"_individual_average_dissimilarity_frequecy_distribution", ".pdf", sep= "" ),
       width=7, height=7, paper="special", fonts="Helvetica")
   mytitle = paste("Frequency distribution of individual average dissimilarity")
   ymax <- max(valcount$count)
   yseq <- c()
   ifelse(ymax <= 10, yseq <- seq(0,ymax+1, 1),
          ifelse(ymax <= 50, yseq <- seq(0,ymax+2, 2),
                 ifelse(ymax <= 100, yseq <- seq(0, ymax+10, 10),
                        ifelse(ymax <= 500, yseq <- seq(0,ymax+50, 50),
                               ifelse(ymax <=1000, yseq <- seq(0, ymax+200, 200),
                                     ifelse(ymax <= 2000, yseq <- seq(0, ymax+400, 400),
                                            ifelse(ymax <= 5000, yseq <- seq(0, ymax+500, 500),
                                                   ifelse(ymax <= 10000, yseq <- seq(0, ymax+1000, 1000),
                                                          ifelse(ymax <= 20000, yseq <- seq(0, ymax+2000, 2000),
                                                                 ifelse(ymax <= 30000, yseq <- seq(0, ymax+3000, 3000),
                                                                        ifelse(ymax <= 40000, yseq <- seq(0, ymax+4000, 4000),
                                                                               ifelse(ymax <= 50000, yseq <- seq(0, ymax+5000, 5000),
                                                                                      ifelse(ymax <= 60000, yseq <- seq(0, ymax+6000, 6000),
                                                                                             ifelse(ymax <= 70000, yseq <- seq(0, ymax+7000, 7000),
                                                                                                    ifelse(ymax <= 80000, yseq <- seq(0, ymax+8000, 8000),
                                                                                                           ifelse(ymax <= 90000, yseq <- seq(0, ymax+9000, 9000),
                                                                                                                  ifelse(ymax <= 100000, yseq <- seq(0, ymax+10000, 10000),
                                                                                                                       yseq <- seq(0, ymax+20000, 20000))))))))))))))))))
   myplot <- barplot(valcount$count, axes=F, main="Frequency distribution of individual average dissimilarity", 
                     xlab =" Average dissimilarity for a sample", ylab = "Sample counts",cex.names= 1.0, ylim=c(0, 1.2*ymax))
   axis(1,at=myplot, labels= mybreaks2)
   axis(2,yseq)
   dev.off()
 }

#-------------------------------------------------------------------
# AMOVA calculation for dominant and haploid data 
#--------------------------------------------------------------------
DT.NA.convert.allele.for.amova <- function(DTMar){    # input just is a Marker DT
   mDToc <- copy(DTMar)
   for (j in names(mDToc)){
     ifelse(Marker_Type == 1L, 
            set(mDToc, which(is.na(mDToc[[j]])), j, "9"),
            ifelse(Marker_Type == 2L | Marker_Type == 3L,        
                   set(mDToc, which(is.na(mDToc[[j]])), j, "0"), 
                   ifelse(Marker_Type == 4L,
                          set(mDToc, which(is.na(mDToc[[j]])), j, "0:0"),  
                          set(mDToc, which(is.na(mDToc[[j]])), j, "00"))))  
   }
   return(mDToc)
 }
 
DT.dom.hap.SSR.SNP.amova.fun <- function(Dt.adind.pop.marker, cs2, NCRDS2){  # Dt.ind.pop.marker is a DT with Ind, Pop, markers
  Dtr <- copy(Dt.adind.pop.marker)                             
  rm(Dt.adind.pop.marker)
  nrw <- Dtr[, .N]
  nc2 <- length(names(Dtr))-2
  loc2n <-Indline.rename(1:nc2, "loc") 
  setkey(Dtr, "Pop")                                           # Dtr is DT with Ind, Pop and markers, sorted by Pop
  
  DT.dom.dis.mat <- function(one.pop.IM, cs2, NCRDS2){
    cs02 <- makeCluster(cs2)    
    registerDoParallel(cs02)     
    setDTthreads(NCRDS2)
    on.exit(stopCluster(cs02))
    DTt <- copy(one.pop.IM)
    MIndts <- headlind.mat.transpose(DTt)
    Marks <- MIndts[, c("Ind"):= NULL]
    dn <- length(names(Marks))
    dr <- Marks[,.N]
    st <-0
    tb <- c()
    tb <- foreach( k = 1:dn, k = 1:dn, .combine= 'c', .inorder=FALSE, .packages=c('foreach','doParallel', 'data.table')) %dopar% {
      d1 <- Marks[, k, with=FALSE]
      a0 <- unlist(d1)
      d2 <- Marks[,-k, with=FALSE]
      hedm <- names(d2)
      mcd <- paste("d",hedm[-1],sep="" )
      d11 <- data.table()
      ifelse(length(hedm)==1L, d11 <- d1,
             d11 <- d1[, c(mcd):= a0])
      setDF(d2)
      setDF(d11)
      d112_ueq <- matrix(as.numeric(d11!=d2), dr, dn-1)
      uequal <-  sum(d112_ueq, na.rm= TRUE)/(dn*2)
      uequal
    }
    st <- sum(tb,na.rm=TRUE)
    return(st)
  }
  
  ssw <- 0
  npm <- unique(Dtr$Pop)
  np <- length(npm)
  nx <- rep(0,np)
  for (np1 in 1:np){
    Dtrp0 <- copy(Dtr)
    setkey(Dtrp0, "Pop")
    Dtrp1 <- Dtrp0[.(npm[np1]), ]
    rm(Dtrp0)
    nn1 <- Dtrp1[, .N ]
    setcolorder(Dtrp1, c("Ind", "Pop", loc2n))
    Dtrp1IM <- Dtrp1[, c("Pop"):= NULL]
    ncount <- DT.dom.dis.mat(Dtrp1IM,cs2,NCRDS2)
    ssw <- ssw + ncount
    nx[np1]<- nn1
  }
  
  keyw <- c("Ind","Pop")
  setkeyv(Dtr, keyw)
  DtrwT <- Dtr[, c("Pop"):= NULL] 
  sst <- DT.dom.dis.mat(DtrwT,cs2,NCRDS2)
  ssa <- sst-ssw
  nx2 <- ((sum(nx))^2-sum(nx^2))/((np-1)*sum(nx))
  nw <- nrw - np
  vw <- ssw/nw
  va <- (ssa/(np-1)-vw)/nx2
  pva <- va/(va+vw)
  pvw <- 1-pva
  amova.out <- cbind(nrw, np, nw, ssa,ssw,va,vw,pva,pvw)
  return(amova.out)
}

#------------------------------------------
# AMOVA calculation for SSR and SNP data
#------------------------------------------
DT.codom.SSR.SNP.amova.fun <- function(Dt.adind.pop.ssr.marker, Matye,cs3, NCRDS3 ){  # ind.pop.snp.marker is a DT with Ind, Pop, markers wide data.table
  
  Dtr <- copy(Dt.adind.pop.ssr.marker)  # Dtr is DT with Ind, Pop and markers, sorted by Pop 
  rm(Dt.adind.pop.ssr.marker)
  nrw <- Dtr[, .N]
  nc2 <- length(names(Dtr))-2
  loc2n <-Indline.rename(1:nc2, "loc") 
  setkey(Dtr, "Pop")
  
  DT.codom.dis.mat <- function(one.pop.indM, Matye, cs3, NCRDS3){
    cs03 <- makeCluster(cs3)                
    registerDoParallel(cs03)                 
    setDTthreads(NCRDS3)
    on.exit(stopCluster(cs03))
    MIndt <- headlind.mat.transpose(one.pop.indM)
    rm(one.pop.indM)
    LN <- MIndt[,1, with=FALSE]
    ln <- Indline.rename(unlist(LN), "Loc")
    LN <- data.table(ln)
    dr <- LN[,.N]
    Markd <- MIndt[, c("Ind"):=NULL]
    rm(MIndt)
    dn <- length(names(Markd))
    st <- 0 
    tb <- c()
    tb <-foreach (k = 1:dn, .combine= 'c', .inorder=FALSE, .packages=c('foreach','doParallel', 'data.table')) %dopar% {
      d1 <- Markd[, k, with=FALSE]
      a3 <- unlist(d1)
      d2 <- Markd[,-k, with=FALSE]
      hdm <- names(d2)
      d2n <- dn-1
      mc <- paste("d",hdm[-1],sep="" )
      d11 <- data.table()
      ifelse(length(hdm)==1L, d11 <- d1,
             d11 <- d1[, c(mc):= a3])
      setDF(d2)
      setDF(d11)
      ud211<- d2!= d11
      ud211[which(ud211==FALSE)]<- NA
      d11s <-matrix(d11[ud211],dr, d2n)
      d2s <- matrix(d2[ud211],dr, d2n)
      rm(d1,d11,d2,ud211)
      DTd11 <- data.table(LN, d11s)
      IDps1 <- paste("ID1", 1:d2n,sep="_")
      setnames(DTd11, c("Loc1",IDps1))
      DTd2 <- data.table(LN, d2s)
      IDps2 <- paste("ID2", 1:d2n,sep="_")
      setnames(DTd2, c("Loc2", IDps2))
      rm(d11s,d2s)
      
      DTd11_melt <- melt(DTd11, measure= list(IDps1), value.name = "alle1" )
      if(isTRUE(Matye==4L)) {DTd11_melt[ , c("p1","p2"):= tstrsplit(alle1, ":", fixed=TRUE)]}
      if(isTRUE(Matye==5L)) {DTd11_melt[ , c("p1","p2"):= tstrsplit(alle1, "", fixed=TRUE)]}
      DTd11_melt[, c("alle1"):= NULL]
      DTd11_meltcast <- dcast(DTd11_melt, Loc1 ~ variable, value.var=c("p1","p2"))
      rm(DTd11_melt)
      DTd11_meltcast[,c("Loc1"):= NULL]
      D1alle1 <- setDF(DTd11_meltcast[, 1:d2n, with=FALSE])
      D1alle2 <- setDF(DTd11_meltcast[, (d2n+1):(2*d2n), with=FALSE])
      AAale <- matrix(as.numeric(D1alle1==D1alle2,na.rm=TRUE), dr,d2n)
      
      DTd2_melt <- melt(DTd2, measure= list(IDps2), value.name = "alle2" )
      if(isTRUE(Matye==4L)) {DTd2_melt[ , c("p1","p2"):= tstrsplit(alle2, ":", fixed=TRUE)]}
      if(isTRUE(Matye==5L)) {DTd2_melt[ , c("p1","p2"):= tstrsplit(alle2, "", fixed=TRUE)]}
      DTd2_melt[, c("alle2"):= NULL]
      DTd2_meltcast <- dcast(DTd2_melt, Loc2 ~ variable, value.var=c("p1","p2"))
      rm(DTd2_melt)
      DTd2_meltcast[,c("Loc2"):= NULL]
      D2alle1 <- setDF(DTd2_meltcast[, 1:d2n, with=FALSE])
      D2alle2 <- setDF(DTd2_meltcast[, (d2n+1):(2*d2n), with=FALSE])
      BBale <- matrix(as.numeric(D2alle1==D2alle2, na.rm=TRUE), dr,d2n)
      FA4 <- setDF(data.table(D1alle1,D1alle1,D1alle2,D1alle2))
      FB4 <- setDF(data.table(D2alle1,D2alle2,D2alle1,D2alle2))
      FAB <- matrix(as.numeric(FA4==FB4,na.rm=TRUE), dr, 4*d2n)
      FABdt <- as.data.table(FAB)
      rm(FA4,FB4,FAB)
      Nme <- c(paste(rep(c("A","B","C","D"), each= d2n), 1:d2n,sep=""))
      setnames(FABdt, Nme)
      FABdt[, c("Loc"):= LN]
      setkey(FABdt, "Loc")
      setcolorder(FABdt, c("Loc", Nme))
      colA <- paste0("A", 1:d2n,sep=""); colB <- paste0("B", 1:d2n,sep=""); colC <- paste0("C", 1:d2n,sep=""); colD <- paste0("D", 1:d2n,sep="") 
      FABdtmelt <- melt(FABdt, measure= list(colA,colB,colC,colD), value.name=c("A1B1", "A1B2","A2B1","A2B2"))
      rm(FABdt,colA,colB,colC,colD)
      FABdtmelt[, c("SS4"):= sum(A1B1, A1B2,A2B1,A2B2), by=c("Loc", "variable")]
      FABdtmelt_dcast <- dcast(FABdtmelt, Loc~variable, value.var = c("SS4"))
      rm(FABdtmelt)
      FABdtmelt_dcast[,c("Loc"):= NULL]
      mixAB <- data.table(LN,AAale,BBale,FABdtmelt_dcast)
      rm(AAale,BBale,FABdtmelt_dcast)
      setnames(mixAB, c("Loc",paste(rep(c("AA","BB","CC"), each= d2n), 1:d2n,sep="")))
      colAA <- paste0("AA", 1:d2n,sep=""); colBB <- paste0("BB", 1:d2n,sep=""); colCC <- paste0("CC", 1:d2n,sep="")
      mixABmelt <- melt(mixAB, measure= list(colAA,colBB,colCC), value.name=c("AA", "BB","CC"))
      rm(mixAB,colAA,colBB,colCC)
      mixABmelt[, c("SS2"):= sum(AA, BB), by=c("Loc", "variable")]
      mixABmelt[SS2==2L & CC== 0L, c("code"):= 4L]
      mixABmelt[SS2==0L & CC== 0L, c("code"):= 2L]
      mixABmelt[SS2==1L & CC== 0L, c("code"):= 3L]
      mixABmelt[is.na(code) & !is.na(CC), c("code"):= 1L]
      mixABmelt[is.na(code) & is.na(CC), c("code"):= 0L]
      mixABmelt_dcast <- dcast(mixABmelt, Loc~variable, value.var = c("code"))
      mixABmelt_dcast[,c("Loc"):= NULL]
      tb1 <- sum(mixABmelt_dcast, na.rm=TRUE)/(2*dn)
      tb1
    }
    st <- sum(tb,na.rm=TRUE)
    return(st)
  }
  
  ssw <- 0
  npm <- unique(Dtr$Pop)
  np <- length(npm)
  nx <- rep(0,np)
  for (np1 in 1:np){     
    Dtrp0 <- copy(Dtr)
    setkey(Dtrp0, "Pop")
    Dtrp1 <- Dtrp0[.(npm[np1]), ]
    rm(Dtrp0)
    nn1 <- Dtrp1[, .N ]
    setcolorder(Dtrp1, c("Ind", "Pop", loc2n))
    #Dtrp1[1:6,1:9,with=FALSE]
    Dtrp1[, c("Pop"):=NULL]
    ncount <- DT.codom.dis.mat(Dtrp1,Matye,cs3, NCRDS3 )
    rm(Dtrp1)
    ssw <- ssw + ncount
    nx[np1]<- nn1
  }
  
  keyw <- c("Ind","Pop")
  setkeyv(Dtr, keyw)
  Dtrwpp <-Dtr[, c("Pop"):=NULL]
  sst <- DT.codom.dis.mat(Dtrwpp, Matye,cs3, NCRDS3)
  ssa <- sst-ssw
  nx2 <- ((sum(nx))^2-sum(nx^2))/((np-1)*sum(nx))
  nw <- nrw - np
  vw <- ssw/nw
  va <- (ssa/(np-1)-vw)/nx2
  pva <- va/(va+vw)
  pvw <- 1-pva
  amova.out <- cbind(nrw, np, nw, ssa,ssw,va,vw,pva,pvw)
  return(amova.out)
}

#-----------------------------------------------------------------------------
# Select high AD sampleS and do amova to assess change in genetic structure
#-----------------------------------------------------------------------------  
DT.simu.high.AD.stepwise.selected.amova <- function(DT.adind.popr.markers, basel, Dyt, anTY,cs4,NCRDS4){
  
  DT.pop.refine <- function(Dtr.ind.pop.mar){   
    Dtorr <- copy(Dtr.ind.pop.mar)
    nrc1 <- length(names(Dtorr))-2
    loc1n <- Indline.rename(1:nrc1, "loc") 
    setkey(Dtorr, "Pop")
    DtorrN <- Dtorr[, .N, by = "Pop"][N>1L]   # filter out the data with only single population for amova analysis
    DtorrN[, "N" := NULL]
    Dtorref <- merge(DtorrN, Dtorr,by = "Pop", all.x=TRUE) 
    setcolorder(Dtorref, c("Ind","Pop",loc1n))
    setkey(Dtorref, "Ind")
    return(Dtorref)
  }
  
  dtp <- copy(DT.adind.popr.markers)
  nrp <- dtp[, .N]
  ncc1 <- length(names(dtp))-2
  locc1n <- Indline.rename(1:ncc1, "loc") 
  setcolorder(dtp, c("Ind", "Pop", locc1n))      # order the column for consistant column names
  setkey(dtp, "Ind")
  nn <- round(nrp/basel, digits = 0)
  fdata <- matrix(NA, nn, 10)
  rowse <- 0
  for (i in 1: nn){
    ifelse (i==1L, rowse1 <- nrp, ifelse(anTY == 1L, rowse1 <- rowse, rowse1 <- nrp-rowse))
    dtps <- copy(dtp)
    dts <- dtps[1:rowse1,]
    setkey(dts,"Pop")
    setcolorder(dts,c("Ind", "Pop", locc1n))
    dts_refine <-DT.pop.refine(dts)
    anr <- dts_refine[, .N]
    amova_out <- numeric(9)
    if (isTRUE(anr >=2L)){
      setkey(dts_refine, "Pop")
      setcolorder(dts_refine,c("Ind", "Pop", locc1n))
      if (isTRUE(Dyt == 1L) | isTRUE(Dyt == 2L) |isTRUE(Dyt == 3L) ) { 
        amova_out<- DT.dom.hap.SSR.SNP.amova.fun(dts_refine, cs4,NCRDS4)
      }
      if (isTRUE(Dyt == 4L) | isTRUE(Dyt == 5L))  {
        amova_out<- DT.codom.SSR.SNP.amova.fun(dts_refine,Dyt,cs4,NCRDS4)
      }
    }
    ifelse (i == 1L, ifelse(anTY == 1L, fdata[1, ] <- c(nrp, amova_out),
                            ifelse(anTY == 2L, fdata[1, ] <- c(rowse, amova_out))),
            fdata[i, ] <- c(rowse, amova_out))
    ifelse(i==1L, rowse <- basel, rowse <- rowse+basel)
  }
  ifelse(anTY==1L, colnames(fdata)<- c("Selected samples", "Samples for AMOVA ", "popN","df","SSa","SSw", "Va","Vw", "PVa","PVw"),
         colnames(fdata)<- c("Removed samples", "Samples for AMOVA", "popN","df","SSa","SSw", "Va","Vw", "PVa","PVw"))
  return(fdata)
}

#---------------------------------------------------------------------------
# labelling function for PCoA plot 
#---------------------------------------------------------------------------
DT.high.AD.select.refine.amova.plot.lable <- function(Ind.Pon, bse, Analty){
  dtt <- copy(Ind.Pon)
  setnames(dtt, c("Ind","Pop"))
  setkey(dtt, "Ind")
  ttn <- dtt[, .N]
  rse <- 0
  tm <- round(ttn/bse, digits = 0)
  selab <- matrix(NA, ttn, tm)
  for (k in 1:tm){
    ifelse(k==1, rse1 <- ttn, ifelse(Analty == 1L, rse1 <- rse, rse1 <- ttn-rse))
    dtw <- copy(dtt)
    dtse <- dtw[1:rse1, ]
    setkey(dtse,"Pop")
    dtseN <- dtse[, .N, by = "Pop"]
    dtsef <- merge(dtseN,dtse,by = "Pop", all = TRUE, allow.cartesian=FALSE)[N>1L]
    dtsef[, "N" := NULL]
    setcolorder(dtsef, c("Ind","Pop"))
    setkey(dtsef, "Ind")
    setkey(dtw, "Ind")
    deme <- merge(dtw, dtsef, all.x =TRUE)
    if (isTRUE(Analty == 1L)){
      deme[!(is.na(Pop.y)), Pop.y1 := "2"]
      deme[is.na(Pop.y), Pop.y1 := "1"]
    }
    if(isTRUE(Analty == 2L)){
      deme[!(is.na(Pop.y)), Pop.y1 := "1"]
      deme[is.na(Pop.y), Pop.y1 := "2"]
    }
    selab[,k]<-deme[,Pop.y1]
    ifelse(k==1, rse <- bse, rse <- rse+bse)
  }
  dtn.lab <- data.table(dtt,selab)
  ifelse(Analty == 1L, setnames(dtn.lab, c("adInd","Pop", paste0("Selected_Label_", 1:tm, sep=""))),
         setnames(dtn.lab, c("adInd","Pop", paste0("Removed_Label_", 1:tm, sep=""))))
  setkey( dtn.lab, "adInd")
  return( dtn.lab)
  }

DT.random.seq.select.high.AD.amova <- function(DT.adind.popr.markers, sel_seq, Dyt, anty, cs5, NCRDS5){
  
  DT.pop.refine <- function(Dtr.ind.pop.mar){   
    Dtorr <- copy(Dtr.ind.pop.mar)
    nrc1 <- length(names(Dtorr))-2
    locnrc1 <- Indline.rename(1:nrc1, "loc")
    setnames(Dtorr, c("Ind","Pop",locnrc1))  
    setkey(Dtorr, "Pop")
    DtorrN <- Dtorr[, .N, by = "Pop"][N>1L]   # filter out the single sample population for AMOVA analysis
    DtorrN[, "N" := NULL]
    Dtorref <- merge(DtorrN, Dtorr,by = "Pop") 
    setcolorder(Dtorref, c("Ind","Pop",locnrc1))
    setkey(Dtorref, "Ind")
    return(Dtorref)
  }
  
  dtp <- copy(DT.adind.popr.markers)
  nrp <- dtp[, .N]
  ncc1 <- length(names(dtp))-2
  locncc1 <- Indline.rename(1:ncc1, "loc")
  setcolorder(dtp, c("Ind", "Pop", locncc1))      # order the column for consistant column names
  setkey(dtp, "Ind")
  ifelse(anty==1L, rowse <- sel_seq, rowse <- nrp-sel_seq)
  rowse <- setdiff(rowse,0)
  nns <- length(rowse)
  fdata <- matrix(NA, nns+1, 10)
  
  for (i in 0:nns){
    ifelse (i==0L, rowse1 <- nrp, rowse1 <- rowse[i])
    dtps <- copy(dtp)
    dts <- dtps[1:rowse1,]
    setkey(dts,"Pop")
    setcolorder(dts,c("Ind", "Pop", locncc1))
    dts_refine <-DT.pop.refine(dts)
    anr <- dts_refine[, .N]
    amova_out <- numeric(9)
    if(isTRUE(anr >= 2)){
      setkey(dts_refine, "Pop")
      setcolorder(dts_refine,c("Ind", "Pop", locncc1))
      
      if (isTRUE(Dyt == 1L) | isTRUE(Dyt == 2L) |isTRUE(Dyt == 3L) ) { 
        amova_out<- DT.dom.hap.SSR.SNP.amova.fun(dts_refine,cs5, NCRDS5 ) 
      }
      if (isTRUE(Dyt == 4L) | isTRUE(Dyt == 5L))  {
        amova_out<- DT.codom.SSR.SNP.amova.fun(dts_refine,Dyt,cs5, NCRDS5 )
      }
    }
    ifelse (i == 0L, ifelse(anty == 1L, fdata[1, ] <- c(nrp, amova_out),
                            ifelse(anty == 2L, fdata[1, ] <- c(nrp-rowse1, amova_out))),
                     ifelse(anty == 1L, fdata[i+1, ] <- c(rowse1, amova_out),
                           ifelse(anty == 2L, fdata[i+1, ] <- c(nrp-rowse1, amova_out))))
  }
  ifelse(anty == 1L, colnames(fdata)<- c("Selected samples", "Samples for AMOVA ", "popN","df","SSa","SSw", "Va","Vw", "PVa","PVw"),
                     colnames(fdata)<- c("Removed samples", "Samples for AMOVA", "popN","df","SSa","SSw", "Va","Vw", "PVa","PVw"))
  return(fdata)
}
 
DT.random.seq.select.high.AD.refine.amova.plot.label <- function(Ind.Pona, bse_seq, nalty){
  dtt <- copy(Ind.Pona)
  setnames(dtt, c("Ind","Pop"))
  setkey(dtt, "Ind")
  ttn <- dtt[, .N]
  ifelse(nalty == 1L, rsel <- bse_seq, rsel <- ttn-bse_seq)
  rsel <- setdiff(rsel,0)
  tm <- length(rsel)
  rse <- 0
  selab <- matrix(NA, ttn, tm+1)
  for (k in 0:tm){
    ifelse(k == 0L, rse <- ttn, rse <- rsel[k])
    dtw <- copy(dtt)
    dtse <- dtw[1:rse, ]
    setkey(dtse,"Pop")
    dtseN <- dtse[, .N, by = "Pop"]
    dtsef <- merge(dtseN,dtse,by = "Pop", all = TRUE, allow.cartesian=FALSE)[N>1L]
    dtsef[, "N" := NULL]
    setcolorder(dtsef, c("Ind","Pop"))
    setkey(dtsef, "Ind")
    setkey(dtw, "Ind")
    deme <- merge(dtw, dtsef, all.x =TRUE)
    if (isTRUE(nalty == 1L)){
      deme[!(is.na(Pop.y)), Pop.y1 := "2"]
      deme[is.na(Pop.y), Pop.y1 := "1"]
    }
    if(isTRUE(nalty == 2L)){
      deme[!(is.na(Pop.y)), Pop.y1 := "1"]
      deme[is.na(Pop.y), Pop.y1 := "2"]
    }
    selab[,k+1]<-deme[,Pop.y1]
  }
  
  dtn.lab <- data.table(dtt, selab)
  ifelse(nalty==1L, setnames(dtn.lab, c("adInd","Pop", paste0("Selected_Label_", 0:tm, sep=""))),
                    setnames(dtn.lab, c("adInd","Pop", paste0("Removed_Label_", 0:tm, sep=""))))
  setkey(dtn.lab, "adInd")
  return(dtn.lab)
  }

selected.PCoA.plot <- function(adPCoAvalues, kipe2, IDpop3,p12,p22,anTy2, marTy2, Pro){
  PCoAv <- copy(adPCoAvalues)
  IDpob2 <- copy(IDpop3)
  ppn <- length(names(IDpob2))-1
  setnames(IDpob2, c("adInd",paste("pop", 1:ppn, sep = "")))
  setkey(IDpob2,"adInd")
  PCoA12 <- PCoAv[, p12, with=FALSE]
  PCoA22 <- PCoAv[, p22, with=FALSE]
  xld12 <- c(min(PCoA12)-0.001, 1.2*max(PCoA12))
  yld12 <- c(min(PCoA22)-0.001, 1.2*max(PCoA22))
  
  for (i in 1:ppn){
    j <- i+1
    popl <-IDpob2[, j, with=FALSE] 
    dat2 <- data.table(PCoA12, PCoA22, popl )
    setnames(dat2, c("ppc1","ppc2", "popl"))
    setkey(dat2, popl)
    nrd <- dat2[,.N]
    popn <- dat2[,.N, by = popl]
    mm <- popn[, N]
    PCs11 <- dat2[, ppc1]
    PCs22 <- dat2[, ppc2]
    selt <- as.character()
    cols <- as.character()
    mycol <-as.character()  
    if(isTRUE(length(mm)==2L)){
      selt <- c("Unselected", "Selected")
      cols <- c("grey", "black")
      mycol <- rep(c("grey","black"), c(mm[1], mm[2]))
    }
    if(anTy2== 1L) {
      jpeg(file=paste(marTy2,"_", anTy2,"_", Pro,"_", j, "_","selected_", mm[2], "_among_", nrd, "_samples_",p12,"_",p22,"_","PCoA", ".jpg", sep= "" ))
      plot(PCs11, PCs22, pch=20, col=mycol, cex = 1.0, xlab = paste("PCoA1"," ", "(",round(kipe2[p12],2), "%",")", sep=" "),
           ylab = paste("PCoA2"," ","(", round(kipe2[p22],2),"%", ")",sep=" "), xlim = xld12, ylim = yld12)
      abline(h = 0, v = 0, col = "gray60", lty=3)
      title(paste("A PCoA plot of ",mm[2], " Selected ","(", "black", ")"," vs ","original samples",  sep=" "))
      legend("topleft", legend=paste(selt), col = cols, pch=16, bty="n",ncol=1, cex=1.0, pt.cex=0.9)
      dev.off()
    }
    if(isTRUE(anTy2== 2L)) {
      jpeg(file=paste(marTy2,"_", anTy2,"_", Pro,"_", j, "_", "removed_", mm[2], "_among_", nrd, "_samples_",p12,"_",p22,"_","PCoA", ".jpg", sep= "" ))
      plot(PCs11, PCs22, pch=20, col=mycol, cex = 1.0, xlab = paste(paste("PCoA", p12, sep="")," ","(", round(kipe2[p12],2), "%", ")",sep=" "),
           ylab = paste(paste("PCoA", p22, sep="")," ","(" ,round(kipe2[p22],2), "%",")", sep=" "), xlim = xld12, ylim = yld12)
      abline(h = 0, v = 0, col = "gray60", lty=3)
      title(paste("A PCoA plot of ",mm[2], " Selected ","(", "black", ")"," vs ","original samples",  sep=" "))
      legend("topleft", legend=paste(selt), col = cols, pch=16, bty="n",ncol=1, cex=1.0, pt.cex=0.9)
      dev.off()
    }
  }
}


##################################### 
# Main program
#####################################

#----------------------------------------------------------------------------------------------------------------
#  1. Raw Inpop_markers used for Ind and Pop format 
#----------------------------------------------------------------------------------------------------------------

Fmark_trans <- Raw.Ind.Pop.rename.Mark.Format.NA.replace(Inpop_markers, Miss_Data, Marker_Type)    
setkey(Fmark_trans, "Ind")
 
# Fmark_trans[, .N]

Fmark_trans1 <- copy(Fmark_trans)
#Fmark_trans1[, .N]
#Fmark_trans1[1:10,1:5,with=FALSE]


Raw_Indx <-Fmark_trans1[, .(Ind)] 
#names(Fmark_trans1)

rIndmar <- Fmark_trans1[, c("Pop","IndR","PopR"):= NULL]           
# names(rIndmar)

rm(Fmark_trans1)

Fu_dissim_mat <- DT.dom.hap.codom.AD.mat(rIndmar, ccs, NCreads) 
rm (rIndmar)

Fu_AD <- apply(Fu_dissim_mat, 1, mean, na.rm = TRUE)
Fu_AD_plot <- AD.count.plot(Fu_AD, Marker_Type, Analysis_Type)       # plot AD distribution
Fu_AD_Ind2_Pop2_Marker_DT <- data.table(Fu_AD, Fmark_trans)          # combine the AD with ind,pop and marker DT
rm(Fmark_trans)

#----------------------------------------------------------------------------------------------------------------
# 2. Calculate PCoA value
#----------------------------------------------------------------------------------------------------------------

Fu_dissim_mat[is.na(Fu_dissim_mat)] <- 0
ann2 <- nrow(Fu_dissim_mat)

Ad_dist <- as.dist(Fu_dissim_mat, upper = FALSE)
Ad_dist <- sqrt(Ad_dist)
res <- cmdscale(Ad_dist, k = (NROW(Fu_dissim_mat) - 1), eig = TRUE, add = TRUE)
eig2 <- eigenvals(res)
kipe <- 100 * eig2/sum(eig2)               # PCoA eigenvalues proportion

PCoA_value_Raw <- res$points
PCoA_vector <- data.table(Raw_Indx,Fu_AD, PCoA_value_Raw[, 1:4])                         # combine the raw Indx, AD with PCoA values
setnames(PCoA_vector, c("Raw_Indx","AD","PCoA1","PCoA2","PCoA3","PCoA4"))
setkey(PCoA_vector, "Raw_Indx") 
PCoA_vector <- PCoA_vector[, lapply(.SD, as.numeric), .SDcols= c("AD","PCoA1","PCoA2","PCoA3","PCoA4"), by = Raw_Indx]
Fu_ad_PCoA_vector0 <- PCoA_vector[order(-AD)]                                             # order -AD from high to low to reorder the PCoA values

#----------------------------------------------------------------------------------------------------------------
# 3. Generate AD descending data file 
#----------------------------------------------------------------------------------------------------------------

des_Fu_AD_Ind3_Pop3_Marker_DT <- DT.AD.desced.index.random(Fu_AD_Ind2_Pop2_Marker_DT)     # output has 6 columns for marker information
rm(Fu_AD_Ind2_Pop2_Marker_DT)

Fu_adInd <- des_Fu_AD_Ind3_Pop3_Marker_DT[, adInd]
Fu_Pop <- des_Fu_AD_Ind3_Pop3_Marker_DT[, Pop]
Fu_IndR <- des_Fu_AD_Ind3_Pop3_Marker_DT[, IndR]
Fu_PopR <- des_Fu_AD_Ind3_Pop3_Marker_DT[, PopR]

Fu_ad_PCoA_vector <- data.table(Fu_adInd, Fu_Pop,Fu_IndR,Fu_PopR,Fu_ad_PCoA_vector0[, -1, with = FALSE])
setnames(Fu_ad_PCoA_vector, c("adInd","Pop","IndR","PopR","AD","PCoA1","PCoA2","PCoA3","PCoA4"))
write.csv(Fu_ad_PCoA_vector, file =paste(Marker_Type,"_", Analysis_Type,"_",stepwise_selection,"_",ann2,
                                  "_Fu_AD_order","_first_4_PCoA_vectors_results",".csv",sep=""))

Fu_adInd_Pop_Markers <- des_Fu_AD_Ind3_Pop3_Marker_DT[, c("AD","Ind","IndR","PopR"):= NULL, ]
rm(des_Fu_AD_Ind3_Pop3_Marker_DT)
locnamc <- Indline.rename(1:Loci_Ns, "loc")

setnames(Fu_adInd_Pop_Markers,c("Ind","Pop",locnamc))   # for AMOVA analysis
setkey(Fu_adInd_Pop_Markers, "Ind")                                              # adInd keyed 

#----------------------------------------------------------------------------------------------------------------
# 4. select high AD samples to assess changes in genetic structure by AMOVA 
#----------------------------------------------------------------------------------------------------------------

Fu_Ad_selected_amova_level_labels_stepwise <- data.table()    # output labels for plot
Fu_Ad_selected_amova_level_labels_specific <- data.table()
Fu_adInd_Pop_Markers1 <- copy(Fu_adInd_Pop_Markers)

Fu_adInd_Pop <- Fu_adInd_Pop_Markers1[, .(Ind, Pop), ]
marDTc <- Fu_adInd_Pop_Markers1[, -c(1:2), with=FALSE] 
marDTco <- DT.NA.convert.allele.for.amova(marDTc)          # convert all NA into a unique allele according to marker type for AMOVA calculation
Fu_adInd_Pop_Markers1 <- data.table(Fu_adInd_Pop, marDTco)
Nss <- Fu_adInd_Pop[, .N]
rm(marDTc, marDTco)

if (isTRUE(!is.na(bas1)) & isTRUE(bas1 > 0L)) {
  bas2 <- bas1 
  Fu_Ad_selected_amova_df <-DT.simu.high.AD.stepwise.selected.amova(Fu_adInd_Pop_Markers1, bas2, Marker_Type, Analysis_Type,ccs,NCreads)
  Fu_Ad_selected_amova_level_labels_stepwise <- DT.high.AD.select.refine.amova.plot.lable(Fu_adInd_Pop, bas2, Analysis_Type)

  if(isTRUE(Analysis_Type== 1L)){
     write.csv(Fu_Ad_selected_amova_df, file=paste(Marker_Type, "_", Analysis_Type, "_", stepwise_selection, "_", Nss,
    	 "_Fu_AD_order_stepwise_selected_amova_results",".csv",sep=""))
   }

  if(isTRUE(Analysis_Type== 2L)){
     write.csv(Fu_Ad_selected_amova_df, file=paste(Marker_Type, "_", Analysis_Type, "_", stepwise_selection, "_", Nss,
    	 "_Fu_AD_order_stepwise_removed_amova_results",".csv",sep=""))
   }
}

if (isTRUE(!is.na(seqlabel)) & isTRUE(seqlabel > 0L)){
  bas_seq <- seqlabel 
  Fu_Ad_selected_amova_op <-DT.random.seq.select.high.AD.amova(Fu_adInd_Pop_Markers1, bas_seq, Marker_Type, Analysis_Type,ccs,NCreads)
  Fu_Ad_selected_amova_level_labels_specific <- DT.random.seq.select.high.AD.refine.amova.plot.label(Fu_adInd_Pop, bas_seq, Analysis_Type)
  
  if(isTRUE(Analysis_Type== 1)){
    write.csv(Fu_Ad_selected_amova_op, file=paste(Marker_Type,"_", Analysis_Type,"_", Nss,
	           "_Fu_AD_order_specific_selected_amova_results",".csv",sep=""))
   }
  
  if(isTRUE(Analysis_Type== 2)){
    write.csv(Fu_Ad_selected_amova_op, file=paste(Marker_Type,"_", Analysis_Type,"_", Nss,
	    "_Fu_AD_order_specific_removed_amova_results",".csv",sep=""))
   }
 }
rm(Fu_adInd_Pop_Markers1)

#----------------------------------------------------------------------------------------------------------------
# 5.  PCoA plot displays 
#----------------------------------------------------------------------------------------------------------------

Fu_ad_PCoAV <- Fu_ad_PCoA_vector[, 6:9, with = FALSE]

if (isTRUE(!is.na(bas1)) & isTRUE(bas1 > 0L)){
  ifelse(length(names(Fu_Ad_selected_amova_level_labels_stepwise))>=6,
              Fu_lables <- Fu_Ad_selected_amova_level_labels_stepwise[,c(1,4:6),with=FALSE],
              Fu_lables <- Fu_Ad_selected_amova_level_labels_stepwise[,-c(2:3),with=FALSE])
   select_plot <- selected.PCoA.plot(Fu_ad_PCoAV, kipe, Fu_lables, 1, 2, Analysis_Type, Marker_Type, stepwise_selection)
}

if (isTRUE(!is.na(seqlabel)) & isTRUE(seqlabel > 0L)){
   Fu_lables <- Fu_Ad_selected_amova_level_labels_specific[, -c(2:3), with=FALSE]
   select_plot <- selected.PCoA.plot(Fu_ad_PCoAV, kipe, Fu_lables, 1, 2, Analysis_Type, Marker_Type, specific_selection)
  }

#----------------------------------------------------------------------------------------------------------------
# 6. Save and move outputs to the results subdirectory
#----------------------------------------------------------------------------------------------------------------

flist1 <- list.files(path = path1, pattern = "*average_dissimilarity*", all.files = FALSE,  full.names = TRUE,
                     recursive = FALSE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE)
ifelse(Analysis_Type==1L, flist2 <- list.files(path = path1, pattern = "*selected_amova*", all.files = FALSE,  full.names = TRUE,
                                                recursive = FALSE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE), 
                        flist2 <- list.files(path = path1, pattern = "*removed_amova*", all.files = FALSE,  full.names = TRUE,
                                                recursive = FALSE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE)) 
flist3 <- list.files(path = path1, pattern = "*PCoA.jp*", all.files = FALSE,  full.names = TRUE,
                     recursive = FALSE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE)
flist4 <- list.files(path = path1, pattern = "*PCoA_vectors*", all.files = FALSE,  full.names = TRUE,
                     recursive = FALSE, ignore.case = FALSE, include.dirs = FALSE, no.. = FALSE)
  
file.copy(from = c(flist1, flist2,  flist3, flist4), to = workpath[work4], overwrite = TRUE, copy.mode = TRUE)
file.remove(c(flist1, flist2, flist3, flist4))

#----------------------------------------------------------------------------------------------------------------
# 7. exit
#----------------------------------------------------------------------------------------------------------------

quit()
 

